# Copyright 2025 Matheus Vilano
# SPDX-License-Identifier: Apache-2.0

from pywwise.objects.effects.mastering_suite import MasteringSuite
from pywwise.objects.effects.wwise_3d_audio_bed_mixer import Wwise3DAudioBedMixer
from pywwise.objects.effects.wwise_compressor import WwiseCompressor
from pywwise.objects.effects.wwise_convolution_reverb import WwiseConvolutionReverb
from pywwise.objects.effects.wwise_delay import WwiseDelay
from pywwise.objects.effects.wwise_expander import WwiseExpander
from pywwise.objects.effects.wwise_flanger import WwiseFlanger
from pywwise.objects.effects.wwise_gain import WwiseGain
from pywwise.objects.effects.wwise_guitar_distortion import WwiseGuitarDistortion
from pywwise.objects.effects.wwise_harmonizer import WwiseHarmonizer
from pywwise.objects.effects.wwise_matrix_reverb import WwiseMatrixReverb
from pywwise.objects.effects.wwise_meter import WwiseMeter
from pywwise.objects.effects.wwise_parametric_eq import WwiseParametricEq
from pywwise.objects.effects.wwise_peak_limiter import WwisePeakLimiter
from pywwise.objects.effects.wwise_pitch_shifter import WwisePitchShifter
from pywwise.objects.effects.wwise_recorder import WwiseRecorder
from pywwise.objects.effects.wwise_reflect import WwiseReflect
from pywwise.objects.effects.wwise_roomverb import WwiseRoomVerb
from pywwise.objects.effects.wwise_stereo_delay import WwiseStereoDelay
from pywwise.objects.effects.wwise_time_stretch import WwiseTimeStretch
from pywwise.objects.effects.wwise_tremolo import WwiseTremolo



# Copyright 2025 Matheus Vilano
# SPDX-License-Identifier: Apache-2.0

from pywwise.descriptors import WwiseProperty
from pywwise.enums import EAudioObjectOptions, EMainMixConfiguration, EPassthroughMixPolicy
from pywwise.objects.abc import WwiseObject


class Wwise3DAudioBedMixer(WwiseObject):
    """
    https://www.audiokinetic.com/en/library/edge/?source=SDK&id=wwiseobject_effect_wwise_3d_audio_bed_mixer.html \n
    A class serving as an interface for getting/setting properties on Wwise objects. This type specifically targets
    the class represented by `EObjectType.WWISE_3D_AUDIO_BED_MIXER`.
    """
    main_mix_configuration = WwiseProperty[EMainMixConfiguration]("MainMixConfiguration", EMainMixConfiguration)
    passthrough_mix_policy = WwiseProperty[EPassthroughMixPolicy]("PassThroughMixPolicy", EPassthroughMixPolicy)
    system_audio_object_limit = WwiseProperty[int]("SystemAudioObjectLimit", int)
    system_audio_objects_policy = WwiseProperty[EAudioObjectOptions]("SystemAudioObjectsPolicy", EAudioObjectOptions)



# Copyright 2025 Matheus Vilano
# SPDX-License-Identifier: Apache-2.0

from pywwise.descriptors import WwiseProperty
from pywwise.objects.abc import WwiseObject
from pywwise.objects.types.effect import Effect
from pywwise.objects.types.rtpc import Rtpc


class EffectSlot(WwiseObject):
    """
    https://www.audiokinetic.com/en/library/edge/?source=SDK&id=wwiseobject_effectslot.html \n
    A class serving as an interface for getting/setting properties on Wwise objects. This type specifically targets
    the class represented by `EObjectType.EFFECT_SLOT`.
    """
    bypass = WwiseProperty[bool]("Bypass", bool)
    effect = WwiseProperty[Effect]("Effect", Effect)
    rtpc = WwiseProperty[tuple[Rtpc, ...]]("RTPC", tuple)
    render = WwiseProperty[bool]("Render", bool)



# Copyright 2025 Matheus Vilano
# SPDX-License-Identifier: Apache-2.0

from pywwise.descriptors import WwiseProperty
from pywwise.enums import EColour, EMusicClipType, ESwitchOrState
from pywwise.objects.abc import WwiseObject


class MusicTrackSequence(WwiseObject):
    """
    https://www.audiokinetic.com/en/library/edge/?source=SDK&id=wwiseobject_musictracksequence.html \n
    A class serving as an interface for getting/setting properties on Wwise objects. This type specifically targets
    the class represented by `EObjectType.MUSIC_TRACK_SEQUENCE`.
    """
    clips = WwiseProperty[tuple[EMusicClipType, ...]]("Clips", tuple)
    colour = WwiseProperty[EColour]("Color", EColour)
    override_colour = WwiseProperty[bool]("OverrideColor", bool)
    track_sequence_switch = WwiseProperty[ESwitchOrState]("TrackSequenceSwitch", ESwitchOrState)



# Copyright 2025 Matheus Vilano
# SPDX-License-Identifier: Apache-2.0

from pywwise.descriptors import WwiseProperty
from pywwise.enums import EColour
from pywwise.objects.abc import WwiseObject


class AcousticTexture(WwiseObject):
    """
    https://www.audiokinetic.com/en/library/edge/?source=SDK&id=wwiseobject_acoustictexture.html \n
    A class serving as an interface for getting/setting properties on Wwise objects. This type specifically targets
    the class represented by `EObjectType.ACOUSTIC_TEXTURE`.
    """
    absorption_high = WwiseProperty[float]("AbsorptionHigh", float)
    absorption_low = WwiseProperty[float]("AbsorptionLow", float)
    absorption_mid_high = WwiseProperty[float]("AbsorptionMidHigh", float)
    absorption_mid_low = WwiseProperty[float]("AbsorptionMidLow", float)
    absorption_offset = WwiseProperty[float]("AbsorptionOffset", float)
    colour = WwiseProperty[EColour]("Color", EColour)
    override_colour = WwiseProperty[bool]("OverrideColor", bool)
    scattering = WwiseProperty[float]("Scattering", float)



# Copyright 2025 Matheus Vilano
# SPDX-License-Identifier: Apache-2.0

from pywwise.descriptors import WwiseProperty
from pywwise.objects.abc import WwiseObject


class WwiseDelay(WwiseObject):
    """
    https://www.audiokinetic.com/en/library/edge/?source=SDK&id=wwiseobject_effect_wwise_delay.html \n
    A class serving as an interface for getting/setting properties on Wwise objects. This type specifically targets
    the class represented by `EObjectType.WWISE_DELAY`.
    """
    delay_time = WwiseProperty[float]("DelayTime", float)
    feedback = WwiseProperty[float]("Feedback", float)
    feedback_enabled = WwiseProperty[bool]("FeedbackEnabled", bool)
    output_level = WwiseProperty[float]("OutputLevel", float)
    process_lfe = WwiseProperty[bool]("ProcessLFE", bool)
    wet_dry_mix = WwiseProperty[float]("WetDryMix", float)



# Copyright 2024 Matheus Vilano
# SPDX-License-Identifier: Apache-2.0

from enum import Enum as _Enum
from re import findall as _re_findall, match as _re_match, split as _re_split
from typing import Any as _Any, Type as _Type, TypeVar as _TypeVar

from pywwise.metas import StaticMeta
from pywwise.modules import LazyModule

_EnumType = _TypeVar("_EnumType", bound=_Enum)
_pywwise_enums = LazyModule("pywwise.enums")  # To avoid circular imports.


class EnumStatics(metaclass=StaticMeta):
    """A static class containing useful utility functions for enums."""
    
    @staticmethod
    def from_value(enum_type: _Type[_EnumType], enum_value: _Any) -> _EnumType:
        """
        Gets an enum member whose value matches the provided value.
        :param enum_type: The enum type to run the search on.
        :param enum_value: The value to search for.
        :raise: ValueError, if the provided value does not exist in the enum type.
        :return: An enum member instance.
        """
        for member in enum_type:
            if member == enum_value:
                return member
        raise ValueError(f"No {_EnumType.__name__} member with value {enum_value}")
    
    @staticmethod
    def get_all_members(enum_type: _Type[_EnumType]) -> tuple[_EnumType, ...]:
        """
        Gets all members of an enum type.
        :param enum_type: The enum type to get all members from.
        :return: A tuple containing all members of an enum type.
        """
        return tuple(member for member in enum_type)


class StringStatics(metaclass=StaticMeta):
    """A static class containing useful utility functions for strings."""
    
    @staticmethod
    def validate_case(text: str) -> bool:
        """
        Checks if the provided string matches one of the common case styles (Snake, Camel, Pascal, etc.). Those are
        enumerated in `pywwise.enums.ECaseStyle`.
        :param text: The string to validate.
        :return: Whether the string is formatted in accordance to one of the common case styles (see
                 `pywwise.enums.ECaseStyle` for a complete list).
        """
        for case in _pywwise_enums.ECaseStyle:
            if _re_match(case, text):
                return True
        return False
    
    @staticmethod
    def to_camel_case(text: str) -> str:
        """
        Converts text to camelCase, to the best possible extent.
        :param text: The string to convert to camelCase.
        :return: The string, with the camelCase format.
        """
        if _re_match(_pywwise_enums.ECaseStyle.CAMEL, text):
            return text
        tokens = _re_split(r"[-_\s]", text)
        if tokens == [text]:  # Handle PascalCase or camelCase
            tokens = _re_findall(r"[A-Z]?[a-z]+|[A-Z]+(?=[A-Z]|$)", text)
        return tokens[0].lower() + ''.join(x.title() for x in tokens[1:])  # Convert to camelCase
    
    @staticmethod
    def to_kebab_case(text: str) -> str:
        """
        Converts text to kebab-case, to the best possible extent.
        :param text: The string to convert to kebab-case.
        :return: The string, with the kebab-case format.
        """
        if _re_match(_pywwise_enums.ECaseStyle.KEBAB, text):
            return text
        if _re_match(_pywwise_enums.ECaseStyle.PASCAL, text) or _re_match(_pywwise_enums.ECaseStyle.CAMEL, text):
            tokens = _re_findall(r"[A-Z]?[a-z]+|[A-Z]+(?=[A-Z]|$)", text)
        else:
            tokens = _re_split(r"[-_\s]", text)
        return '-'.join([token.lower() for token in tokens])
    
    @staticmethod
    def to_pascal_case(text: str) -> str:
        """
        Converts text to PascalCase, to the best possible extent.
        :param text: The string to convert to PascalCase.
        :return: The string, with the PascalCase format.
        """
        if _re_match(_pywwise_enums.ECaseStyle.PASCAL, text):
            return text
        as_camel = StringStatics.to_camel_case(text)
        return f"{as_camel[0].upper()}{as_camel[1:]}"
    
    @staticmethod
    def to_snake_case(text: str) -> str:
        """
        Converts text to snake_case, to the best possible extent.
        :param text: The string to convert to snake_case.
        :return: The string, with the snake_case format.
        """
        if _re_match(_pywwise_enums.ECaseStyle.SNAKE, text):
            return text
        return StringStatics.to_kebab_case(text).replace('-', '_')
    
    @staticmethod
    def to_upper_case(text: str) -> str:
        """
        Converts text to UPPER_CASE, to the best possible extent.
        :param text: The string to convert to UPPER_CASE.
        :return: The string, with the UPPER_CASE format.
        """
        if _re_match(_pywwise_enums.ECaseStyle.UPPER, text):
            return text
        return StringStatics.to_snake_case(text).upper()


class JsonStatics(metaclass=StaticMeta):
    """A static class containing useful utility functions for dictionaries that represent JSON objects."""
    
    @staticmethod
    def from_object(obj: _Any) -> dict:
        """
        Creates a JSON-like dictionary based on the provided object's `__dict__`.
        :param obj: The object to use when creating the dictionary.
        :return: A JSON-like dictionary representing the provided object.
        :raise: NotImplementedError - this function is tentative and is still in the works.
        """
        raise NotImplementedError("This function is not implemented yet. For this functionality, try using the "
                                  "`dictionary` property of structs, if available.")



# Copyright 2025 Matheus Vilano
# SPDX-License-Identifier: Apache-2.0

from pywwise.descriptors import WwiseProperty
from pywwise.enums import EActionType, EScope, ESeekType, ESetterType
from pywwise.objects.abc import WwiseObject
from pywwise.objects.types.effect import Effect
from pywwise.objects.types.effect_slot import EffectSlot


class Action(WwiseObject):
    """
    https://www.audiokinetic.com/en/library/edge/?source=SDK&id=wwiseobject_action.html \n
    A class serving as an interface for getting/setting properties on Wwise objects. This type specifically targets
    the class represented by `EObjectType.ACTION`.
    """
    absolute_or_relative = WwiseProperty[ESetterType]("AbsoluteOrRelative", ESetterType)
    action_type = WwiseProperty[EActionType]("ActionType", EActionType)
    apply_to_dynamic_sequence = WwiseProperty[bool]("ApplyToDynamicSequence", bool)
    apply_to_state_transition = WwiseProperty[bool]("ApplyToStateTransition", bool)
    bypass_flag = WwiseProperty[bool]("BypassFlag", bool)
    bypass_game_parameter_internal_transition = WwiseProperty[bool]("BypassGameParameterInternalTransition", bool)
    delay = WwiseProperty[float]("Delay", float)
    effect_slot = WwiseProperty[EffectSlot]("EffectSlot", EffectSlot)
    fade_in_curve = WwiseProperty[int]("FadeInCurve", int)
    fade_out_curve = WwiseProperty[int]("FadeOutCurve", int)
    fade_time = WwiseProperty[float]("FadeTime", float)
    game_parameter_value = WwiseProperty[float]("GameParameterValue", float)
    highpass = WwiseProperty[int]("Highpass", int)
    inclusion = WwiseProperty[bool]("Inclusion", bool)
    lowpass = WwiseProperty[int]("Lowpass", int)
    master_resume = WwiseProperty[bool]("MasterResume", bool)
    pause_delayed_resume_action = WwiseProperty[bool]("PauseDelayedResumeAction", bool)
    pitch = WwiseProperty[float]("Pitch", float)
    probability = WwiseProperty[float]("Probability", float)
    resume_state_transition = WwiseProperty[bool]("ResumeStateTransition", bool)
    scope = WwiseProperty[EScope]("Scope", EScope)
    seek_percent = WwiseProperty[float]("SeekPercent", float)
    seek_time = WwiseProperty[float]("SeekTime", float)
    seek_to_marker = WwiseProperty[bool]("SeekToMarker", bool)
    seek_type = WwiseProperty[ESeekType]("SeekType", ESeekType)
    target = WwiseProperty[WwiseObject]("Target", WwiseObject)
    target_effect = WwiseProperty[Effect]("TargetEffect", Effect)
    volume = WwiseProperty[float]("Volume", float)



# Copyright 2025 Matheus Vilano
# SPDX-License-Identifier: Apache-2.0

from __future__ import annotations

from typing import TYPE_CHECKING, Union as _Union

if TYPE_CHECKING:
    from pywwise.objects.types.music_transition import MusicTransition

from pywwise.descriptors import WwiseProperty
from pywwise.enums import (E3DPosition, E3DSpatialization, EColour, EDiscardBehaviour, ELoudnessNormalizationType,
                           EMidiTempoSource, EMusicTrackType, EOverLimitBehaviour, EScope, ESpeakerPanning,
                           ESwitchOrState, ESyncGroupType, EVirtualVoiceBehaviour, EVirtualVoiceQueueBehaviour)
from pywwise.objects.abc import WwiseObject
from pywwise.objects.types.attenuation import Attenuation
from pywwise.objects.types.aux_bus import AuxBus
from pywwise.objects.types.blend_container import BlendContainer
from pywwise.objects.types.bus import Bus
from pywwise.objects.types.conversion import Conversion
from pywwise.objects.types.effect_slot import EffectSlot
from pywwise.objects.types.music_track_sequence import MusicTrackSequence
from pywwise.objects.types.random_sequence_container import RandomSequenceContainer
from pywwise.objects.types.rtpc import Rtpc
from pywwise.objects.types.sound import Sound
from pywwise.objects.types.switch_container import SwitchContainer
from pywwise.primitives import GUID

_MidiTargetType = _Union[RandomSequenceContainer, SwitchContainer, BlendContainer, Sound]

_MidiTargetTypeTuple = (RandomSequenceContainer, SwitchContainer, BlendContainer, Sound)


class MusicTrack(WwiseObject):
    """
    https://www.audiokinetic.com/en/library/edge/?source=SDK&id=wwiseobject_musictrack.html \n
    A class serving as an interface for getting/setting properties on Wwise objects. This type specifically targets
    the class represented by `EObjectType.MUSIC_TRACK`.
    """
    position_3d = WwiseProperty[E3DPosition]("3DPosition", E3DPosition)
    spatialization_3d = WwiseProperty[E3DSpatialization]("3DSpatialization", E3DSpatialization)
    attenuation = WwiseProperty[Attenuation]("Attenuation", Attenuation)
    attenuation_distance_scaling = WwiseProperty[float]("AttenuationDistanceScaling", float)
    below_threshold_behaviour = WwiseProperty[EVirtualVoiceBehaviour]("BelowThresholdBehavior", EVirtualVoiceBehaviour)
    bypass_effect = WwiseProperty[bool]("BypassEffect", bool)
    centre_percentage = WwiseProperty[float]("CenterPercentage", float)
    colour = WwiseProperty[EColour]("Color", EColour)
    conversion = WwiseProperty[Conversion]("Conversion", Conversion)
    default_switch_or_state = WwiseProperty[ESwitchOrState]("DefaultSwitchOrState", ESwitchOrState)
    effects = WwiseProperty[tuple[EffectSlot, ...]]("Effects", tuple)
    enable_attenuation = WwiseProperty[bool]("EnableAttenuation", bool)
    enable_diffraction = WwiseProperty[bool]("EnableDiffraction", bool)
    enable_loudness_normalization = WwiseProperty[bool]("EnableLoudnessNormalization", bool)
    game_aux_send_hpf = WwiseProperty[int]("GameAuxSendHPF", int)
    game_aux_send_lpf = WwiseProperty[int]("GameAuxSendLPF", int)
    game_aux_send_volume = WwiseProperty[float]("GameAuxSendVolume", float)
    hdr_active_range = WwiseProperty[float]("HdrActiveRange", float)
    hdr_enable_envelope = WwiseProperty[bool]("HdrEnableEnvelope", bool)
    hdr_envelope_sensitivity = WwiseProperty[float]("HdrEnvelopeSensitivity", float)
    highpass = WwiseProperty[int]("Highpass", int)
    hold_emitter_position_orientation = WwiseProperty[bool]("HoldEmitterPositionOrientation", bool)
    hold_listener_orientation = WwiseProperty[bool]("HoldListenerOrientation", bool)
    ignore_parent_max_sound_instance = WwiseProperty[bool]("IgnoreParentMaxSoundInstance", bool)
    inclusion = WwiseProperty[bool]("Inclusion", bool)
    is_global_limit = WwiseProperty[EScope]("IsGlobalLimit", EScope)
    is_non_cachable = WwiseProperty[bool]("IsNonCachable", bool)
    is_streaming_enabled = WwiseProperty[bool]("IsStreamingEnabled", bool)
    is_zero_latency = WwiseProperty[bool]("IsZeroLatency", bool)
    listener_relative_routing = WwiseProperty[bool]("ListenerRelativeRouting", bool)
    look_ahead_time = WwiseProperty[int]("LookAheadTime", int)
    loudness_normalization_target = WwiseProperty[float]("LoudnessNormalizationTarget", float)
    loudness_normalization_type = WwiseProperty[ELoudnessNormalizationType](
        "LoudnessNormalizationType", ELoudnessNormalizationType)
    lowpass = WwiseProperty[int]("Lowpass", int)
    make_up_gain = WwiseProperty[float]("MakeUpGain", float)
    max_reached_behaviour = WwiseProperty[EDiscardBehaviour]("MaxReachedBehavior", EDiscardBehaviour)
    max_sound_per_instance = WwiseProperty[int]("MaxSoundPerInstance", int)
    metadata = WwiseProperty[GUID]("Metadata", GUID)
    midi_target = WwiseProperty[_MidiTargetType]("MidiTarget", _MidiTargetTypeTuple)
    midi_tempo_source = WwiseProperty[EMidiTempoSource]("MidiTempoSource", EMidiTempoSource)
    music_track_type = WwiseProperty[EMusicTrackType]("MusicTrackType", EMusicTrackType)
    output_bus = WwiseProperty[Bus]("OutputBus", Bus)
    output_bus_highpass = WwiseProperty[int]("OutputBusHighpass", int)
    output_bus_lowpass = WwiseProperty[int]("OutputBusLowpass", int)
    output_bus_volume = WwiseProperty[float]("OutputBusVolume", float)
    over_limit_behaviour = WwiseProperty[EOverLimitBehaviour]("OverLimitBehavior", EOverLimitBehaviour)
    override_analysis = WwiseProperty[bool]("OverrideAnalysis", bool)
    override_colour = WwiseProperty[bool]("OverrideColor", bool)
    override_conversion = WwiseProperty[bool]("OverrideConversion", bool)
    override_early_reflections = WwiseProperty[bool]("OverrideEarlyReflections", bool)
    override_effect = WwiseProperty[bool]("OverrideEffect", bool)
    override_game_aux_sends = WwiseProperty[bool]("OverrideGameAuxSends", bool)
    override_hdr_envelope = WwiseProperty[bool]("OverrideHdrEnvelope", bool)
    override_metadata = WwiseProperty[bool]("OverrideMetadata", bool)
    override_midi_target = WwiseProperty[bool]("OverrideMidiTarget", bool)
    override_midi_tempo = WwiseProperty[bool]("OverrideMidiTempo", bool)
    override_output = WwiseProperty[bool]("OverrideOutput", bool)
    override_positioning = WwiseProperty[bool]("OverridePositioning", bool)
    override_priority = WwiseProperty[bool]("OverridePriority", bool)
    override_user_aux_sends = WwiseProperty[bool]("OverrideUserAuxSends", bool)
    override_virtual_voice = WwiseProperty[bool]("OverrideVirtualVoice", bool)
    pro_fetch_length = WwiseProperty[int]("ProFetchLength", int)
    priority = WwiseProperty[int]("Priority", int)
    priority_distance_factor = WwiseProperty[bool]("PriorityDistanceFactor", bool)
    priority_distance_offset = WwiseProperty[int]("PriorityDistanceOffset", int)
    rtpc = WwiseProperty[tuple[Rtpc, ...]]("RTPC", tuple)
    reflections_aux_send = WwiseProperty[AuxBus]("ReflectionsAuxSend", AuxBus)
    reflections_volume = WwiseProperty[float]("ReflectionsVolume", float)
    sequences = WwiseProperty[tuple[MusicTrackSequence, ...]]("Sequences", tuple)
    speaker_panning = WwiseProperty[ESpeakerPanning]("SpeakerPanning", ESpeakerPanning)
    speaker_panning_3d_spatialization_mix = WwiseProperty[int]("SpeakerPanning3DSpatializationMix", int)
    switch_or_state_group = WwiseProperty[ESyncGroupType]("SwitchOrStateGroup", ESyncGroupType)
    transition_root: WwiseProperty[MusicTransition]  # Injected via __init__
    use_game_aux_sends = WwiseProperty[bool]("UseGameAuxSends", bool)
    use_max_sound_per_instance = WwiseProperty[bool]("UseMaxSoundPerInstance", bool)
    user_aux_send_0 = WwiseProperty[AuxBus]("UserAuxSend0", AuxBus)
    user_aux_send_1 = WwiseProperty[AuxBus]("UserAuxSend1", AuxBus)
    user_aux_send_2 = WwiseProperty[AuxBus]("UserAuxSend2", AuxBus)
    user_aux_send_3 = WwiseProperty[AuxBus]("UserAuxSend3", AuxBus)
    user_aux_send_hpf_0 = WwiseProperty[int]("UserAuxSendHPF0", int)
    user_aux_send_hpf_1 = WwiseProperty[int]("UserAuxSendHPF1", int)
    user_aux_send_hpf_2 = WwiseProperty[int]("UserAuxSendHPF2", int)
    user_aux_send_hpf_3 = WwiseProperty[int]("UserAuxSendHPF3", int)
    user_aux_send_lpf_0 = WwiseProperty[int]("UserAuxSendLPF0", int)
    user_aux_send_lpf_1 = WwiseProperty[int]("UserAuxSendLPF1", int)
    user_aux_send_lpf_2 = WwiseProperty[int]("UserAuxSendLPF2", int)
    user_aux_send_lpf_3 = WwiseProperty[int]("UserAuxSendLPF3", int)
    user_aux_send_volume_0 = WwiseProperty[float]("UserAuxSendVolume0", float)
    user_aux_send_volume_1 = WwiseProperty[float]("UserAuxSendVolume1", float)
    user_aux_send_volume_2 = WwiseProperty[float]("UserAuxSendVolume2", float)
    user_aux_send_volume_3 = WwiseProperty[float]("UserAuxSendVolume3", float)
    virtual_voice_queue_behaviour = WwiseProperty[EVirtualVoiceQueueBehaviour](
        "VirtualVoiceQueueBehavior", EVirtualVoiceQueueBehaviour)
    volume = WwiseProperty[float]("Volume", float)



# Copyright 2025 Matheus Vilano
# SPDX-License-Identifier: Apache-2.0

from pywwise.descriptors import WwiseProperty
from pywwise.objects.abc import WwiseObject


class WwiseExpander(WwiseObject):
    """
    https://www.audiokinetic.com/en/library/edge/?source=SDK&id=wwiseobject_effect_wwise_expander.html \n
    A class serving as an interface for getting/setting properties on Wwise objects. This type specifically targets
    the class represented by `EObjectType.WWISE_EXPANDER`.
    """
    attack_time = WwiseProperty[float]("AttackTime", float)
    channel_link = WwiseProperty[bool]("ChannelLink", bool)
    output_gain = WwiseProperty[float]("OutputGain", float)
    process_lfe = WwiseProperty[bool]("ProcessLFE", bool)
    ratio = WwiseProperty[float]("Ratio", float)
    release_time = WwiseProperty[float]("ReleaseTime", float)
    threshold = WwiseProperty[float]("Threshold", float)



# Copyright 2025 Matheus Vilano
# SPDX-License-Identifier: Apache-2.0

from pywwise.descriptors import WwiseProperty
from pywwise.enums import EFrequencyMode, ENoiseColour, ESynthOneOperationMode, ESynthOneWaveform
from pywwise.objects.abc import WwiseObject


class WwiseSynthOne(WwiseObject):
    """
    https://www.audiokinetic.com/en/library/edge/?source=SDK&id=wwiseobject_source_wwise_synth_one.html \n
    A class serving as an interface for getting/setting properties on Wwise objects. This type specifically targets
    the class represented by `EObjectType.SOURCE_PLUGIN`.
    """
    base_frequency = WwiseProperty[float]("BaseFrequency", float)
    fm_amount = WwiseProperty[float]("FMAmount", float)
    frequency_mode = WwiseProperty[EFrequencyMode]("FrequencyMode", EFrequencyMode)
    noise_level = WwiseProperty[ENoiseColour]("NoiseLevel", ENoiseColour)
    noise_shape = WwiseProperty[float]("NoiseShape", float)
    operation_mode = WwiseProperty[ESynthOneOperationMode]("OperationMode", ESynthOneOperationMode)
    osc1_invert = WwiseProperty[bool]("Osc1Invert", bool)
    osc1_level = WwiseProperty[float]("Osc1Level", float)
    osc1_pwm = WwiseProperty[float]("Osc1Pwm", float)
    osc1_transpose = WwiseProperty[float]("Osc1Transpose", float)
    osc1_waveform = WwiseProperty[ESynthOneWaveform]("Osc1Waveform", ESynthOneWaveform)
    osc2_invert = WwiseProperty[bool]("Osc2Invert", bool)
    osc2_level = WwiseProperty[float]("Osc2Level", float)
    osc2_pwm = WwiseProperty[float]("Osc2Pwm", float)
    osc2_transpose = WwiseProperty[float]("Osc2Transpose", float)
    osc2_waveform = WwiseProperty[ESynthOneWaveform]("Osc2Waveform", ESynthOneWaveform)
    output_level = WwiseProperty[float]("OutputLevel", float)
    oversampling = WwiseProperty[bool]("Oversampling", bool)



# Copyright 2025 Matheus Vilano
# SPDX-License-Identifier: Apache-2.0

from __future__ import annotations

from typing import TYPE_CHECKING as _TYPE_CHECKING

if _TYPE_CHECKING:
    from pywwise.objects.types.music_segment import MusicSegment

from pywwise.descriptors import WwiseProperty
from pywwise.enums import EColour, EMusicSegmentPlayPoint
from pywwise.objects.abc import WwiseObject
from pywwise.objects.types.trigger import Trigger


class MusicStinger(WwiseObject):
    """
    https://www.audiokinetic.com/en/library/edge/?source=SDK&id=wwiseobject_musicstinger.html \n
    A class serving as an interface for getting/setting properties on Wwise objects. This type specifically targets
    the class represented by `EObjectType.MUSIC_STINGER`.
    """
    colour = WwiseProperty[EColour]("Color", EColour)
    custom_cue_match_name = WwiseProperty[str]("CustomCueMatchName", str)
    dont_play_again_time = WwiseProperty[float]("DontPlayAgainTime", float)
    num_segment_advance = WwiseProperty[bool]("NumSegmentAdvance", bool)
    override_colour = WwiseProperty[bool]("OverrideColor", bool)
    play_segment_at = WwiseProperty[EMusicSegmentPlayPoint]("PlaySegmentAt", EMusicSegmentPlayPoint)
    segment: WwiseProperty[MusicSegment]  # Injected via __init__
    trigger = WwiseProperty[Trigger]("Trigger", Trigger)
    


# Copyright 2025 Matheus Vilano
# SPDX-License-Identifier: Apache-2.0

from typing import Self as _Self

from pywwise.descriptors import WwiseProperty
from pywwise.enums import (E3DPosition, E3DSpatialization, EBusChannelConfiguration, EColour)
from pywwise.objects.abc import WwiseObject
from pywwise.objects.types.attenuation import Attenuation
from pywwise.objects.types.effect_slot import EffectSlot
from pywwise.objects.types.rtpc import Rtpc
from pywwise.primitives import GUID


class AuxBus(WwiseObject):
    """
    https://www.audiokinetic.com/en/library/edge/?source=SDK&id=wwiseobject_auxbus.html \n
    A class serving as an interface for getting/setting properties on Wwise objects. This type specifically targets
    the class represented by `EObjectType.AUX_BUS`.
    """
    position_3d = WwiseProperty[E3DPosition]("3DPosition", E3DPosition)
    spatialization_3d = WwiseProperty[E3DSpatialization]("3DSpatialization", E3DSpatialization)
    attenuation = WwiseProperty[Attenuation]("Attenuation", Attenuation)
    attenuation_distance_scaling = WwiseProperty[float]("AttenuationDistanceScaling", float)
    bus_channel_config = WwiseProperty[EBusChannelConfiguration]("BusChannelConfig", EBusChannelConfiguration)
    bus_volume = WwiseProperty[float]("BusVolume", float)
    bypass_all_effects = WwiseProperty[bool]("BypassEffect", bool)
    centre_percentage = WwiseProperty[float]("CenterPercentage", float)
    colour = WwiseProperty[EColour]("Color", EColour)
    effects = WwiseProperty[tuple[EffectSlot, ...]]("Effects", tuple)
    enable_attenuation = WwiseProperty[bool]("EnableAttenuation", bool)
    enable_diffraction = WwiseProperty[bool]("EnableDiffraction", bool)
    game_aux_send_hpf = WwiseProperty[int]("GameAuxSendHPF", int)
    game_aux_send_lpf = WwiseProperty[int]("GameAuxSendLPF", int)
    game_aux_send_volume = WwiseProperty[float]("GameAuxSendVolume", float)
    highpass = WwiseProperty[int]("Highpass", int)
    hold_emitter_position_orientation = WwiseProperty[bool]("HoldEmitterPositionOrientation", bool)
    hold_listener_orientation = WwiseProperty[bool]("HoldListenerOrientation", bool)
    listener_relative_routing = WwiseProperty[bool]("ListenerRelativeRouting", bool)
    lowpass = WwiseProperty[int]("Lowpass", int)
    metadata = WwiseProperty[GUID]("Metadata", GUID)
    output_bus_highpass = WwiseProperty[int]("OutputBusHighpass", int)
    output_bus_lowpass = WwiseProperty[int]("OutputBusLowpass", int)
    output_bus_volume = WwiseProperty[float]("OutputBusVolume", float)
    override_colour = WwiseProperty[bool]("OverrideColor", bool)
    pitch = WwiseProperty[int]("Pitch", int)
    rtpc = WwiseProperty[tuple[Rtpc, ...]]("RTPC", tuple)
    reflections_aux_send = WwiseProperty[_Self]("ReflectionsAuxSend", _Self)
    reflections_volume = WwiseProperty[float]("ReflectionsVolume", float)
    speaker_panning = WwiseProperty[int]("SpeakerPanning", int)
    speaker_panning_3d_spatialization_mix = WwiseProperty[int]("SpeakerPanning3DSpatializationMix", int)
    use_game_aux_sends = WwiseProperty[bool]("UseGameAuxSends", bool)
    user_aux_send_0 = WwiseProperty[_Self]("UserAuxSend0", _Self)
    user_aux_send_1 = WwiseProperty[_Self]("UserAuxSend1", _Self)
    user_aux_send_2 = WwiseProperty[_Self]("UserAuxSend2", _Self)
    user_aux_send_3 = WwiseProperty[_Self]("UserAuxSend3", _Self)
    user_aux_send_hpf_0 = WwiseProperty[int]("UserAuxSendHPF0", int)
    user_aux_send_hpf_1 = WwiseProperty[int]("UserAuxSendHPF1", int)
    user_aux_send_hpf_2 = WwiseProperty[int]("UserAuxSendHPF2", int)
    user_aux_send_hpf_3 = WwiseProperty[int]("UserAuxSendHPF3", int)
    user_aux_send_lpf_0 = WwiseProperty[int]("UserAuxSendLPF0", int)
    user_aux_send_lpf_1 = WwiseProperty[int]("UserAuxSendLPF1", int)
    user_aux_send_lpf_2 = WwiseProperty[int]("UserAuxSendLPF2", int)
    user_aux_send_lpf_3 = WwiseProperty[int]("UserAuxSendLPF3", int)
    user_aux_send_volume_0 = WwiseProperty[float]("UserAuxSendVolume0", float)
    user_aux_send_volume_1 = WwiseProperty[float]("UserAuxSendVolume1", float)
    user_aux_send_volume_2 = WwiseProperty[float]("UserAuxSendVolume2", float)
    user_aux_send_volume_3 = WwiseProperty[float]("UserAuxSendVolume3", float)
    voice_volume = WwiseProperty[float]("Volume", float)



# Copyright 2025 Matheus Vilano
# SPDX-License-Identifier: Apache-2.0

from pywwise.descriptors import WwiseProperty
from pywwise.enums import EMechanismPlayMode, ERandomOrSequence
from pywwise.objects.abc import WwiseObject
from pywwise.objects.types.rtpc import Rtpc


class Position(WwiseObject):
    """
    https://www.audiokinetic.com/en/library/edge/?source=SDK&id=wwiseobject_position.html \n
    A class serving as an interface for getting/setting properties on Wwise objects. This type specifically targets
    the class represented by `EObjectType.POSITION`.
    """
    new_path_for_each_sound = WwiseProperty[bool]("NewPathForEachSound", bool)
    pan_x = WwiseProperty[float]("PanX", float)
    pan_y = WwiseProperty[float]("PanY", float)
    pan_z = WwiseProperty[float]("PanZ", float)
    play_mechanism_loop = WwiseProperty[bool]("PlayMechanismLoop", bool)
    play_mechanism_random_or_sequence = WwiseProperty[ERandomOrSequence](
        "PlayMechanismRandomOrSequence", ERandomOrSequence)
    play_mechanism_step_or_continuous = WwiseProperty[EMechanismPlayMode]("PlayMechanismStepOrContinuous", EMechanismPlayMode)
    play_mechanism_transition_time = WwiseProperty[float]("PlayMechanismTransitionTime", float)
    play_mechanism_transition_use = WwiseProperty[bool]("PlayMechanismTransitionUse", bool)
    rtpc = WwiseProperty[tuple[Rtpc, ...]]("RTPC", tuple)



# Copyright 2024 Matheus Vilano
# SPDX-License-Identifier: Apache-2.0

from simplevent import RefEvent as _RefEvent
from waapi import WaapiClient as _WaapiClient

from pywwise.decorators import callback
from pywwise.enums import EReturnOptions
from pywwise.primitives import GUID, ProjectPath
from pywwise.structs import SwitchContainerAssignment, WwiseObjectInfo


class SwitchContainer:
    """ak.wwise.core.switchContainer"""
    
    def __init__(self, client: _WaapiClient):
        """
        Constructor.
        :param client: The WAAPI client to use.
        """
        self._client = client
        
        assignment_args = {"return": [EReturnOptions.GUID, EReturnOptions.NAME,
                                      EReturnOptions.TYPE, EReturnOptions.PATH]}
        
        self.assignment_added = _RefEvent(WwiseObjectInfo, WwiseObjectInfo, WwiseObjectInfo)
        """
        https://www.audiokinetic.com/library/edge/?source=SDK&id=ak_wwise_core_switchcontainer_assignmentadded.html
        \nSent when an assignment is added to a Switch Container.
        \n**Event Data**:
        \n- A WwiseObjectInfo instance representing the Switch Container where a new assignment was added.
        \n- A WwiseObjectInfo instance representing the child object that was assigned.
        \n- A WwiseObjectInfo instance representing the State or Switch to which the child object was assigned.
        """
        
        self._assignment_added = self._client.subscribe("ak.wwise.core.switchContainer.assignmentAdded",
                                                        self._on_assignment_added, assignment_args)
        
        self.assignment_removed = _RefEvent(WwiseObjectInfo, WwiseObjectInfo, WwiseObjectInfo)
        """
        https://www.audiokinetic.com/library/edge/?source=SDK&id=ak_wwise_core_switchcontainer_assignmentadded.html
        \nSent when an assignment is removed from a Switch Container.
        \n**Event Data**:
        \n- A WwiseObjectInfo instance representing the Switch Container where an assignment was removed.
        \n- A WwiseObjectInfo instance representing the child object that was part of the removed assignment.
        \n- A WwiseObjectInfo instance representing the State or Switch to which the child object was assigned.
        """
        
        self._assignment_added = self._client.subscribe("ak.wwise.core.switchContainer.assignmentRemoved",
                                                        self._on_assignment_removed, assignment_args)
    
    @callback
    def _on_assignment_added(self, event: _RefEvent, **kwargs):
        """
        Callback function for the `assignmentAdded` event.
        :param event: The event to broadcast.
        :param kwargs: The event data.
        """
        event(*self._on_assignment_changed(**kwargs))
    
    @callback
    def _on_assignment_removed(self, event: _RefEvent, **kwargs):
        """
        Callback function for the `assignmentRemoved` event.
        :param event: The event to broadcast.
        :param kwargs: The event data.
        """
        event(*self._on_assignment_changed(**kwargs))
    
    @staticmethod
    def _on_assignment_changed(**kwargs) -> tuple[WwiseObjectInfo, WwiseObjectInfo, WwiseObjectInfo]:
        """
        Utility function for the `assignmentAdded` and `assignmentRemoved` events.
        :param kwargs: The event data.
        :return: The event data, processed.
        """
        container = WwiseObjectInfo.from_dict(kwargs["switchContainer"])
        child = WwiseObjectInfo.from_dict(kwargs["child"])
        sync = WwiseObjectInfo.from_dict(kwargs["stateOrSwitch"])
        return container, child, sync
    
    def add_assignment(self, child: GUID | ProjectPath, state_or_switch: GUID | ProjectPath) -> bool:
        """
        https://www.audiokinetic.com/library/edge/?source=SDK&id=ak_wwise_core_switchcontainer_addassignment.html \n
        Assigns a Switch Container's child to a State or Switch. This is the equivalent of doing a drag-and-drop of
        the child to a state or switch in the Assigned Objects view. The child is always added at the end for each
        state.
        :param child: The GUID or project path of the object to assign to a State or Switch. This object must be a
                      child of a Switch Container.
        :param state_or_switch: The GUID or path of the State or Switch with which to assign. Must be the child of the
                                Switch Group or State Group that is currently set for the Switch Container.
        :return: Whether the call succeeded. Will return `False` if the assignment already exists.
        """
        args = {"child": child, "stateOrSwitch": state_or_switch}
        return self._client.call("ak.wwise.core.switchContainer.addAssignment", args) is not None
    
    def get_assignments(self, switch_container: GUID | ProjectPath) -> tuple[SwitchContainerAssignment, ...]:
        """
        https://www.audiokinetic.com/library/edge/?source=SDK&id=ak_wwise_core_switchcontainer_getassignments.html \n
        Returns the list of assignments between a Switch Container's children and states.
        :param switch_container: The GUID or project path of the Switch Container from which to get assignments.
        :return: A tuple of `SwitchContainerAssignment` instances. May be empty (if there are no valid assignments or
                 if the call failed).
        """
        results = self._client.call("ak.wwise.core.switchContainer.getAssignments", {"id": switch_container})
        if results is None:
            return ()
        results = results.get("return", ())
        return tuple(SwitchContainerAssignment(result["child"], result["stateOrSwitch"]) for result in results)
    
    def remove_assignment(self, child: GUID | ProjectPath, state_or_switch: GUID | ProjectPath) -> bool:
        """
        https://www.audiokinetic.com/library/edge/?source=SDK&id=ak_wwise_core_switchcontainer_removeassignment.html \n
        Removes an assignment between a Switch Container's child and a State or Switch.
        :param child: The GUID or project path of the object assigned to State or Switch. This object must be a child
                      of a Switch Container.
        :param state_or_switch: The GUID or path of the State or Switch in the assignment. Must be the child of the
                                Switch Group or State Group that is currently set for the Switch Container.
        :return: Whether the call succeeded. Will return `False` if the assignment does not exist.
        """
        args = {"child": child, "stateOrSwitch": state_or_switch}
        return self._client.call("ak.wwise.core.switchContainer.removeAssignment", args) is not None



# Copyright 2025 Matheus Vilano
# SPDX-License-Identifier: Apache-2.0


from pywwise.descriptors import WwiseProperty
from pywwise.enums import EColour
from pywwise.objects.abc import WwiseObject


class State(WwiseObject):
    """
    https://www.audiokinetic.com/en/library/edge/?source=SDK&id=wwiseobject_state.html \n
    A class serving as an interface for getting/setting properties on Wwise objects. This type specifically targets
    the class represented by `EObjectType.STATE`.
    """
    colour = WwiseProperty[EColour]("Color", EColour)
    override_colour = WwiseProperty[bool]("OverrideColor", bool)



# Copyright 2024 Matheus Vilano
# SPDX-License-Identifier: Apache-2.0

from abc import ABC as _ABC, abstractmethod as _abstractmethod
from typing import Self as _Self
from uuid import UUID as _UUID


class _PyWwiseType(_ABC):
    """The base class for PyWwise core types."""
    
    @classmethod
    @_abstractmethod
    def get_null(cls) -> _Self:
        """Use when the intention is to represent an "invalid" instance."""
        pass
    
    def is_valid(self) -> bool:
        """
        Checks whether the instance contains a valid value.
        :return: `True`, if the value is valid; else, `False`.
        """
        return self != self.get_null()  # or greater than or equal to 0


class _PyWwiseID(int, _PyWwiseType):
    """The base class for `int` subclasses in PyWwise which represent numeric identifiers."""
    
    def __new__(cls, value: int) -> _Self:
        """
        Creates a new ID.
        :param value: The ID.
        :return: The new ID.
        :raise ValueError: If the ID is less than `0` and not `-1`.
        """
        value = int(value)  # Sometimes the number is a string. In that case, try converting.
        if value < 0 and value != -1:
            raise ValueError("ID value must be non-negative (or -1, if representing an invalid ID).")
        return int.__new__(cls, value)
    
    @classmethod
    def get_null(cls) -> _Self:
        """
        Use when the intention is to represent an "invalid" ID.
        :return: `-1`, which represents an invalid ID.
        """
        return cls(-1)


class _PyWwiseStr(str, _PyWwiseType):
    """The base class for `str` subclasses in PyWwise which represent alphanumeric values."""
    
    def __new__(cls, chars: str) -> _Self:
        """
        Instantiates a new PyWwiseStr.
        :param chars: The string to use to create a new PyWwiseStr.
        :return: The new PyWwiseStr.
        """
        return str.__new__(cls, chars)
    
    @classmethod
    def get_null(cls) -> _Self:
        """
        Gets a "null" instance. Use this to represent ideas such as "invalidity" or "nonexistence".
        :return: A "null" instance, which contains a single character: the null-terminator (`\0`).
        """
        return cls("\0")


class _PyWwisePath(_PyWwiseStr):
    
    def __new__(cls, path: str, windows_style: bool = True) -> _Self:
        """
        Creates a new path-like object using a `str` as the container. By default, any path will be automatically
        converted to a Windows-style path. Example: `"/Actor-Mixer Hierarchy/Default Work Unit"` will be converted to
        `"\\Actor-Mixer Hierarchy\\Default Work Unit"`. This behaviour can be overridden.
        :param path: The path string.
        :param windows_style: Whether the path should be forced to a Windows-style format. Example: if `True`,
                              `"/Actor-Mixer Hierarchy/Default Work Unit"` will be converted to
                              `"\\Actor-Mixer Hierarchy\\Default Work Unit"`.
        :return: A new `PyWwisePath` object.
        :raise ValueError: If the path is empty.
        """
        delimiter = "\\" if windows_style else "/"
        
        if path == "\0":
            path = super().__new__(cls, path)
            path._delimiter = delimiter
            return path
        
        if windows_style:
            path = path.replace("/", "\\")
        else:
            path = path.replace("\\", "/")
        
        if not path:  # Empty!
            raise ValueError("The provided path is invalid or empty. Must be a valid path-like string.")
        
        if path[0] != delimiter:
            path = f"{delimiter}{path}"
        
        if path[-1] == delimiter:
            path = path[:-1]
        
        path = super().__new__(cls, path)
        path._delimiter = delimiter
        return path
    
    def __getitem__(self, item: int | slice) -> str | list[str]:
        """
        Gets one or more components from the path.
        :param item: The index or slice to use.
        :return: One or more components from the path.
        """
        return self.split(self._delimiter)[1:][item]
    
    def __truediv__(self, path: str) -> _Self:
        """
        Build or extend a path using the `/` operator.
        :param path: The path, as a string.
        :return: The new path.
        """
        if isinstance(path, str):
            path = self.__class__(f"{self}{self._delimiter}{path.lstrip('/')}")
            return path.replace("/" if self._delimiter == "\\" else "/", self._delimiter)
        else:
            raise TypeError(f"Unsupported type for path join: {type(path)}")


class Name(_PyWwiseStr):
    """A Wwise object Name. This is usually intended to be used with unique objects (e.g. State Groups)."""
    
    def __new__(cls, name: str) -> _Self:
        """
        Instantiates a new Name.
        :param name: The string to use to create a new Name.
        :return: The new Name.
        :raises ValueError: If the name is empty.
        """
        if len(name) <= 0:
            raise ValueError("The provided name string is empty.")
        return str.__new__(cls, name)


class GUID(_PyWwiseStr):
    """A Wwise object GUID (e.g. `"{63726145-57FB-490B-B611-738BD3EF2F72}"`."""
    
    def __new__(cls, guid: str) -> _Self:
        """
        Instantiates a new GUID.
        :param guid: The string to use to create a new GUID.
        :return: The new GUID.
        :raise ValueError: If the GUID is in the wrong format.
        """
        if (guid[0] != '{' or guid[-1] != '}') or not cls.validate(guid):
            raise ValueError(f"Wrong GUID format. Expected format is: \"{cls.get_null()}\" (alphanumerical).")
        return str.__new__(cls, guid)
    
    @classmethod
    def get_null(cls) -> _Self:
        """
        A GUID containing "zero" (the default "invalid" GUID).
        :return: The GUID "{00000000-0000-0000-0000-000000000000}".
        """
        return GUID("{00000000-0000-0000-0000-000000000000}")
    
    @classmethod
    def validate(cls, value: str) -> bool:
        """
        Validates the provided string as a GUID.
        :param value: The string to validate as a GUID.
        :return: Whether the provided string is a valid GUID.
        """
        try:
            _UUID(value, version=4)
            return True
        except ValueError:
            return False


class ProjectPath(_PyWwisePath):
    """A project path (e.g. `"\\Actor-Mixer Hierarchy\\Default Work Unit\\MyActorMixer"`)."""
    
    def __new__(cls, path: str, windows_style: bool = True):
        """
        Creates a new path-like object using a `str` as the container. By default, any path will be automatically
        converted to a Windows-style path. Example: `"/Actor-Mixer Hierarchy/Default Work Unit"` will be converted to
        `"\\Actor-Mixer Hierarchy\\Default Work Unit"`. This behaviour can be overridden.
        :param path: The path string.
        :param windows_style: Whether the path should be forced to a Windows-style format. Example: if `True`,
                              `"/Actor-Mixer Hierarchy/Default Work Unit"` will be converted to
                              `"\\Actor-Mixer Hierarchy\\Default Work Unit"`.
        :return: A new `PyWwisePath` object.
        :raise ValueError: If the path is empty.
        """
        return super().__new__(cls, path)
    
    @classmethod
    def actor_mixer_hierarchy(cls, default_work_unit: bool = False) -> _Self:
        """
        Get path to the default "Actor-" folder. May include the Default Work Unit.
        :param default_work_unit: Whether to include the Default Work Unit in the path.
        :return: '\\Actor-Mixer Hierarchy' or '\\Actor-Mixer Hierarchy\\Default Work Unit'.
        """
        return cls(rf"\Actor-Mixer Hierarchy\{'Default Work Unit' if default_work_unit else ''}")
    
    @classmethod
    def attenuations(cls, default_work_unit: bool = False) -> _Self:
        """
        Get path to the default "Attenuations'" folder. May include the Default Work Unit.
        :param default_work_unit: Whether to include the Default Work Unit in the path.
        :return: '\\Attenuations' or '\\Attenuations\\Default Work Unit'.
        """
        return cls(rf"\Attenuations\{'Default Work Unit' if default_work_unit else ''}")
    
    @classmethod
    def conversion_settings(cls, default_work_unit: bool = False) -> _Self:
        """
        Get path to the default "Conversion Settings" folder. May include the Default Work Unit.
        :param default_work_unit: Whether to include the Default Work Unit in the path.
        :return: '\\Conversion Settings' or '\\Conversion Settings\\Default Work Unit'.
        """
        return cls(rf"\Conversion Settings\{'Default Work Unit' if default_work_unit else ''}")
    
    @classmethod
    def audio_devices(cls, default_work_unit: bool = False) -> _Self:
        """
        Get path to the default "Audio Devices" folder. May include the Default Work Unit.
        :param default_work_unit: Whether to include the Default Work Unit in the path.
        :return: '\\Audio Devices' or '\\Audio Devices\\Default Work Unit'.
        """
        return cls(rf"\Audio Devices\{'Default Work Unit' if default_work_unit else ''}")
    
    @classmethod
    def control_surface_sessions(cls, default_work_unit: bool = False) -> _Self:
        """
        Get path to the default "Control Surface" folder. May include the Default Work Unit.
        :param default_work_unit: Whether to include the Default Work Unit in the path.
        :return: '\\Control Surface Sessions' or '\\Control Surface Sessions\\Default Work Unit'.
        """
        return cls(rf"\Control Surface Sessions\{'Default Work Unit' if default_work_unit else ''}")
    
    @classmethod
    def dynamic_dialogue(cls, default_work_unit: bool = False) -> _Self:
        """
        Get path to the default "Dynamic Dialogue" folder. May include the Default Work Unit.
        :param default_work_unit: Whether to include the Default Work Unit in the path.
        :return: '\\Dynamic Dialogue' or '\\Dynamic Dialogue\\Default Work Unit'.
        """
        return cls(rf"\Dynamic Dialogue\{'Default Work Unit' if default_work_unit else ''}")
    
    @classmethod
    def effects(cls, default_work_unit: bool = False) -> _Self:
        """
        Get path to the default "Effects'" folder. May include the Default Work Unit.
        :param default_work_unit: Whether to include the Default Work Unit in the path.
        :return: '\\Effects' or '\\Effects\\Default Work Unit'.
        """
        return cls(rf"\Effects\{'Default Work Unit' if default_work_unit else ''}")
    
    @classmethod
    def events(cls, default_work_unit: bool = False) -> _Self:
        """
        Get path to the default "Events'" folder. May include the Default Work Unit.
        :param default_work_unit: Whether to include the Default Work Unit in the path.
        :return: '\\Events' or '\\Events\\Default Work Unit'.
        """
        return cls(rf"\Events\{'Default Work Unit' if default_work_unit else ''}")
    
    @classmethod
    def game_parameters(cls, default_work_unit: bool = False) -> _Self:
        """
        Get path to the default "Game Parameters" folder. May include the Default Work Unit.
        :param default_work_unit: Whether to include the Default Work Unit in the path.
        :return: '\\Game Parameters' or '\\Game Parameters\\Default Work Unit'.
        """
        return cls(rf"\Game Parameters\{'Default Work Unit' if default_work_unit else ''}")
    
    @classmethod
    def interactive_music_hierarchy(cls, default_work_unit: bool = False) -> _Self:
        """
        Get path to the default "Interactive Music" folder. May include the Default Work Unit.
        :param default_work_unit: Whether to include the Default Work Unit in the path.
        :return: '\\Interactive Music Hierarchy' or '\\Interactive Music Hierarchy\\Default Work Unit'.
        """
        return cls(rf"\Interactive Music Hierarchy\{'Default Work Unit' if default_work_unit else ''}")
    
    @classmethod
    def master_mixer_hierarchy(cls, default_work_unit: bool = False) -> _Self:
        """
        Get path to the default "Master-" folder. May include the Default Work Unit.
        :param default_work_unit: Whether to include the Default Work Unit in the path.
        :return: '\\Master-Mixer Hierarchy' or '\\Master-Mixer Hierarchy\\Default Work Unit'.
        """
        return cls(rf"\Master-Mixer Hierarchy\{'Default Work Unit' if default_work_unit else ''}")
    
    @classmethod
    def metadata(cls, default_work_unit: bool = False) -> _Self:
        """
        Get path to the default "Metadata'" folder. May include the Default Work Unit.
        :param default_work_unit: Whether to include the Default Work Unit in the path.
        :return: '\\Metadata' or '\\Metadata\\Default Work Unit'.
        """
        return cls(rf"\Metadata\{'Default Work Unit' if default_work_unit else ''}")
    
    @classmethod
    def mixing_sessions(cls, default_work_unit: bool = False) -> _Self:
        """
        Get path to the default "Mixing Sessions" folder. May include the Default Work Unit.
        :param default_work_unit: Whether to include the Default Work Unit in the path.
        :return: '\\Mixing Sessions' or '\\Mixing Sessions\\Default Work Unit'.
        """
        return cls(rf"\Mixing Sessions\{'Default Work Unit' if default_work_unit else ''}")
    
    @classmethod
    def modulators(cls, default_work_unit: bool = False) -> _Self:
        """
        Get path to the default "Modulators'" folder. May include the Default Work Unit.
        :param default_work_unit: Whether to include the Default Work Unit in the path.
        :return: '\\Modulators' or '\\Modulators\\Default Work Unit'.
        """
        return cls(rf"\Modulators\{'Default Work Unit' if default_work_unit else ''}")
    
    @classmethod
    def queries(cls, default_work_unit: bool = False) -> _Self:
        """
        Get path to the default "Queries'" folder. May include the Default Work Unit.
        :param default_work_unit: Whether to include the Default Work Unit in the path.
        :return: '\\Queries' or '\\Queries\\Default Work Unit'.
        """
        return cls(rf"\Queries\{'Default Work Unit' if default_work_unit else ''}")
    
    @classmethod
    def sound_banks(cls, default_work_unit: bool = False) -> _Self:
        """
        Get path to the default "SoundBanks'" folder. May include the Default Work Unit.
        :param default_work_unit: Whether to include the Default Work Unit in the path.
        :return: '\\SoundBanks' or '\\SoundBanks\\Default Work Unit'.
        """
        return cls(rf"\SoundBanks\{'Default Work Unit' if default_work_unit else ''}")
    
    @classmethod
    def soundcaster_sessions(cls, default_work_unit: bool = False) -> _Self:
        """
        Get path to the default "Soundcaster Sessions" folder. May include the Default Work Unit.
        :param default_work_unit: Whether to include the Default Work Unit in the path.
        :return: '\\Soundcaster Sessions' or '\\Soundcaster Sessions\\Default Work Unit'.
        """
        return cls(rf"\Soundcaster Sessions\{'Default Work Unit' if default_work_unit else ''}")
    
    @classmethod
    def states(cls, default_work_unit: bool = False) -> _Self:
        """
        Get path to the default "States'" folder. May include the Default Work Unit.
        :param default_work_unit: Whether to include the Default Work Unit in the path.
        :return: '\\States' or '\\States\\Default Work Unit'.
        """
        return cls(rf"\States\{'Default Work Unit' if default_work_unit else ''}")
    
    @classmethod
    def switches(cls, default_work_unit: bool = False) -> _Self:
        """
        Get path to the default "Switches'" folder. May include the Default Work Unit.
        :param default_work_unit: Whether to include the Default Work Unit in the path.
        :return: '\\Switches' or '\\Switches\\Default Work Unit'.
        """
        return cls(rf"\Switches\{'Default Work Unit' if default_work_unit else ''}")
    
    @classmethod
    def triggers(cls, default_work_unit: bool = False) -> _Self:
        """
        Get path to the default "Triggers'" folder. May include the Default Work Unit.
        :param default_work_unit: Whether to include the Default Work Unit in the path.
        :return: '\\Triggers' or '\\Triggers\\Default Work Unit'.
        """
        return cls(rf"\Triggers\{'Default Work Unit' if default_work_unit else ''}")
    
    @classmethod
    def virtual_acoustics(cls, default_work_unit: bool = False) -> _Self:
        """
        Get path to the default "Virtual Acoustics" folder. May include the Default Work Unit.
        :param default_work_unit: Whether to include the Default Work Unit in the path.
        :return: '\\Virtual Acoustics' or '\\Virtual Acoustics\\Default Work Unit'.
        """
        return cls(rf"\Virtual Acoustics\{'Default Work Unit' if default_work_unit else ''}")


class OriginalsPath(_PyWwisePath):
    """A source file path, relative to the Originals folder. Note: the Originals path can be customized in Wwise."""
    
    def __new__(cls, path: str, windows_style: bool = True) -> _Self:
        """
        Creates a new OriginalsPath. You can think of this as a string container.
        :param path: The source file path, relative to the Originals folder.
        :param windows_style: Whether the path should be forced to a Windows-style format. Example: if `True`,
                      `"/SFX/Ambience"` will be converted to `"\\SFX\\Ambience"`.
        :return: A new OriginalsPath.
        :raise ValueError: If the path is empty.
        """
        return super().__new__(cls, path)
    
    @classmethod
    def default_sfx(cls) -> _Self:
        """
        Get path to the default "SFX'" folder.
        :return: '\\SFX'.
        """
        return cls(rf"\SFX")
    
    @classmethod
    def default_voices(cls) -> _Self:
        """
        Get path to the default "Voices'" folder.
        :return: '\\Voices'.
        """
        return cls(rf"\Voices")


class GameObjectID(_PyWwiseID):
    """A Game Object ID. This is expected to be a non-negative number."""
    
    @classmethod
    def get_global(cls) -> _Self:
        """
        Specialized factory function. Useful for scripts that target all game objects.
        :return: A new GameObjectID containing the default Global game object ID.
        """
        return GameObjectID((1 << 64) - 1)  # That expression equals the max uint64 value.
    
    @classmethod
    def get_transport(cls) -> _Self:
        """
        Specialized factory function. Useful for scripts where the target game object is Wwise's default transport.
        :return: A new GameObjectID containing the default Transport game object ID.
        """
        return GameObjectID((1 << 64) - 2)  # That expression equals the max uint64 value - 1.


class ShortID(_PyWwiseID):
    """A Wwise object short ID. This is expected to be a non-negative number."""


class PlayingID(_PyWwiseID):
    """A Playing ID, which represents an instance generated from an Event. A negative means the ID is invalid."""



# Copyright 2024 Matheus Vilano
# SPDX-License-Identifier: Apache-2.0

from logging import CRITICAL as _LEVEL_CRITICAL, getLogger as _getLogger

from waapi import CallbackExecutor, SequentialThreadExecutor

from pywwise.aliases import *
from pywwise.enums import *
from pywwise.objects import *
from pywwise.primitives import *
from pywwise.structs import *
from pywwise.waapi.ak import Ak as _Ak, WwiseConnection
from pywwise.waql import *

_getLogger("waapi").setLevel(_LEVEL_CRITICAL)


def new_waapi_connection(url: str = "ws://127.0.0.1:8080/waapi", *, allow_exception: bool = False,
                         callback_executor: CallbackExecutor = SequentialThreadExecutor,
                         is_debug_build: bool = False, is_console_instance: bool = False,
                         watch_list: tuple[WwiseObjectWatch, ...] = ()) -> WwiseConnection:
    """
    Connects to an instance of Wwise.
    :param url: URL of the Wwise Authoring API WAMP server, defaults to `ws://127.0.0.1:8080/waapi`.
    :param allow_exception: Allow errors on call and subscribe to throw an exception. Default is False.
    :param callback_executor: Executor strategy for event callbacks.
    :param is_debug_build: Should be set to true if the instance of Wwise is a debug build and debug-only.
    :param is_console_instance: Should be set to true if the instance of Wwise is running in a console window.
                                functions/topics are required.
    :param watch_list: A tuple of `WwiseObjectWatch` instances. This will be used to set up the
                       `ak.wwise.core.object.property_changed` event.
    :return: A WAAPI connection.
    """
    return _Ak(url, allow_exception, callback_executor, is_debug_build, is_console_instance, watch_list)


def set_waapi_logging(level: int):
    """
    Sets the WAAPI logger's level. By default, the logger will only log `CRITICAL` errors, which are rare. Use this
    function if you are experiencing issues with any functions/topics and would like more information from the logger.
    :param level: The level, represented as an integer. Common values: NOTSET=0, DEBUG=10, INFO=20, WARNING=30,
                  ERROR=40, CRITICAL=50.
    """
    _getLogger("waapi").setLevel(level)



# Copyright 2024 Matheus Vilano
# SPDX-License-Identifier: Apache-2.0

from waapi import WaapiClient as _WaapiClient

from pywwise.enums import EObjectType
from pywwise.primitives import GUID, Name, ProjectPath
from pywwise.structs import PlatformInfo


class Sound:
    """ak.wwise.core.sound"""
    
    def __init__(self, client: _WaapiClient):
        """
        Constructor.
        :param client: The WAAPI client to use.
        """
        self._client = client
    
    def set_active_source(self, sound: GUID | Name | ProjectPath, source: GUID | Name | ProjectPath,
                          platform: PlatformInfo | Name | GUID = None) -> bool:
        """
        https://www.audiokinetic.com/library/edge/?source=SDK&id=ak_wwise_core_sound_setactivesource.html \n
        Sets which version of the source is being used for the specified sound. Use `ak.wwise.core.object.get` with the
        'activeSource' return option to get the active source of a sound.
        :param sound: The GUID or the project path of the sound for which to set the active source.
        :param source: The GUID or the project path of the source.
        :param platform: The platform on which to execute this operation. By default, the active platform in the Wwise
                         instance will be used.
        :return: Whether the call succeeded.
        """
        args = {"sound": f"{EObjectType.SOUND}{sound}" if isinstance(sound, Name) else sound,
                "source": f"{EObjectType.AUDIO_SOURCE}{source}" if isinstance(source, Name) else source}
        if platform is not None:
            args["platform"] = platform
        return self._client.call("ak.wwise.core.sound.setActiveSource", args) is not None



# Copyright 2024 Matheus Vilano
# SPDX-License-Identifier: Apache-2.0

from pywwise.waapi.ak.wwise.core.audio import Audio
from pywwise.waapi.ak.wwise.core.audio_source_peaks import AudioSourcePeaks
from pywwise.waapi.ak.wwise.core.capture_log import CaptureLog
from pywwise.waapi.ak.wwise.core.core import Core
from pywwise.waapi.ak.wwise.core.game_parameter import GameParameter
from pywwise.waapi.ak.wwise.core.log import Log
from pywwise.waapi.ak.wwise.core.object import Object
from pywwise.waapi.ak.wwise.core.profiler import Profiler
from pywwise.waapi.ak.wwise.core.project import Project
from pywwise.waapi.ak.wwise.core.remote import Remote
from pywwise.waapi.ak.wwise.core.sound import Sound
from pywwise.waapi.ak.wwise.core.soundbank import SoundBank
from pywwise.waapi.ak.wwise.core.source_control import SourceControl
from pywwise.waapi.ak.wwise.core.switch_container import SwitchContainer
from pywwise.waapi.ak.wwise.core.transport import Transport
from pywwise.waapi.ak.wwise.core.undo import Undo



# Copyright 2024 Matheus Vilano
# SPDX-License-Identifier: Apache-2.0

from simplevent import RefEvent as _RefEvent
from waapi import WaapiClient as _WaapiClient

from pywwise.aliases import SystemPath
from pywwise.decorators import callback


class Project:
    """ak.wwise.core.project"""
    
    def __init__(self, client: _WaapiClient):
        """
        Constructor.
        :param client: The WAAPI client to use.
        """
        self._client = client
        
        self.loaded = _RefEvent()
        """
        https://www.audiokinetic.com/library/edge/?source=SDK&id=ak_wwise_core_project_loaded.html
        \nSent when the project has been successfully loaded.
        """
        
        self._loaded = self._client.subscribe("ak.wwise.core.project.loaded", self._on_loaded)
        
        self.post_closed = _RefEvent()
        """
        https://www.audiokinetic.com/library/edge/?source=SDK&id=ak_wwise_core_project_postclosed.html
        \nSent when the after the project is completely closed.
        """
        
        self._post_closed = self._client.subscribe("ak.wwise.core.project.postClosed", self._on_post_closed)
        
        self.pre_closed = _RefEvent()
        """
        https://www.audiokinetic.com/library/edge/?source=SDK&id=ak_wwise_core_project_preclosed.html
        \nSent when the project begins closing.
        """
        
        self._pre_closed = self._client.subscribe("ak.wwise.core.project.preClosed", self._on_pre_closed)
        
        self.saved = _RefEvent(tuple[SystemPath, ...])
        """
        https://www.audiokinetic.com/library/edge/?source=SDK&id=ak_wwise_core_project_saved.html
        \nSent when the project has been saved.
        \n**Event Data**:
        \n- A tuple containing the absolute paths to the Work Unit and Project files that were modified.
        """
        
        self._saved = self._client.subscribe("ak.wwise.core.project.saved", self._on_saved)
    
    @callback
    def _on_loaded(self, event: _RefEvent):
        """
        Callback function for the `loaded` event.
        :param event: The event to broadcast.
        """
        event()
    
    @callback
    def _on_post_closed(self, event: _RefEvent):
        """
        Callback function for the `postClosed` event.
        :param event: The event to broadcast.
        """
        event()
    
    @callback
    def _on_pre_closed(self, event: _RefEvent):
        """
        Callback function for the `preClosed` event.
        :param event: The event to broadcast.
        """
        event()
    
    @callback
    def _on_saved(self, event: _RefEvent, **kwargs):
        """
        Callback function for the `saved` event.
        :param event: The event to broadcast.
        :param kwargs: The event data.
        """
        event(tuple(SystemPath(path) for path in kwargs.get("modifiedPaths", dict())))
    
    def save(self, version_control_auto_checkout: bool = True) -> bool:
        """
        https://www.audiokinetic.com/library/edge/?source=SDK&id=ak_wwise_core_project_save.html \n
        Saves the current project.
        :param version_control_auto_checkout: Whether to automatically check out changes to version control. Only
                                              supported in Wwise 2023 or above.
        :return: Whether the operation succeeded.
        """
        args = {"autoCheckOutToSourceControl": False} if not version_control_auto_checkout else dict()
        return self._client.call("ak.wwise.core.project.save", args) is not None



# Copyright 2024 Matheus Vilano
# SPDX-License-Identifier: Apache-2.0

from __future__ import annotations

from typing import Any as _Any, Self as _Self, TYPE_CHECKING as _TYPE_CHECKING

if _TYPE_CHECKING:
    from pywwise.descriptors import WwiseProperty

from dataclasses import dataclass as _dataclass, field as _field
from types import NoneType as _NoneType

from pywwise.aliases import ListOrTuple, SystemPath
from pywwise.enums import (EAttenuationCurveShape, EAttenuationCurveType, EAttenuationCurveUsage, EAudioObjectOptions,
                           EBasePlatform, EBusOptions, ECaptureLogItemType, ECaptureLogSeverity,
                           EGeneratedSoundBankType, EInclusionFilter,
                           ELogSeverity, EObjectType, EReturnOptions, ERtpcMode, EStartMode,
                           EVoicePipelineReturnOptions, EWwiseBuildConfiguration, EWwiseBuildPlatform)
from pywwise.primitives import GameObjectID, GUID, Name, OriginalsPath, PlayingID, ProjectPath, ShortID
from pywwise.statics import EnumStatics


@_dataclass
class Vector2:
    """A 2-dimensional vector."""
    
    x: float
    """X axis."""
    
    y: float
    """Y axis."""
    
    @classmethod
    def get_zero(cls) -> _Self:
        """:return: A Vector3 instance with x, y, and z all set to 0.0."""
        return Vector2(0.0, 0.0)


@_dataclass
class Vector3(Vector2):
    """A 3-dimensional vector."""
    
    z: float
    """Z axis."""
    
    @classmethod
    def get_zero(cls) -> _Self:
        """:return: A Vector3 instance with x, y, and z all set to 0.0."""
        return Vector3(0.0, 0.0, 0.0)


@_dataclass
class Rect:
    """Data-only representation of a rectangle. In some contexts, the position (x, y) can be discarded/ignored."""
    
    x: int
    """Left position of capture region."""
    
    y: int
    """Top position of capture region."""
    
    width: int
    """Width of capture region."""
    
    height: int
    """Height of capture region."""
    
    @staticmethod
    def get_zero():
        """:return: A CaptureRect instance with x, y, width, and height all set to 0."""
        return Rect(0, 0, 0, 0)


@_dataclass
class AuxSendValue:
    """Data-only class representing an aux send value."""
    
    listener: GameObjectID
    """The ID of the associated listener."""
    
    aux_bus: GUID | Name | ShortID
    """The GUID, name, or short ID of the Aux Bus."""
    
    control_value: float
    """The intended value."""


@_dataclass
class PlatformInfo:
    """Structure for storing basic platform info. Useful when creating a new project or adding a new platform to a project."""
    
    name: str | Name
    """The name of this platform."""
    
    base: EBasePlatform
    """The base platform."""
    
    guid: GUID = None
    """The GUID of this platform."""
    
    sound_bank_path: SystemPath = None
    """The path on which the SoundBank files are generated for this platform."""
    
    copied_media_path: SystemPath = None
    """The path on which the SoundBank media files are generated for this platform."""
    
    def __hash__(self):
        """:return: The PlatformInfo hash."""
        return hash(self.name)


@_dataclass
class ExternalSourceInfo:
    """Data-only class storing information about an external source."""
    
    input: SystemPath
    """The path where the external source's WAV file is located."""
    
    platform: Name | GUID
    """The name or GUID of the platform this external source is associated with."""
    
    output: SystemPath
    """The output path of the external source's WEM (after conversions)."""
    
    @property
    def dictionary(self) -> dict[str, str]:
        as_dict = {"input": str(self.input), "platform": str(self.platform)}
        if self.output is not None:
            as_dict["output"] = str(self.output)
        return as_dict


@_dataclass
class WwiseObjectInfo:
    """Data-only class storing core information about a Wwise object."""
    
    guid: GUID
    """The GUID of the wwise object."""
    
    name: Name
    """The name of the Wwise object. Depending on the type, it may be unique."""
    
    type: EObjectType
    """The Wwise object type."""
    
    path: ProjectPath
    """The project path of the Wwise object."""
    
    other: dict[EReturnOptions | str, _Any] = _field(default_factory=dict)
    """A dictionary containing other information, if any. Keys are always strings, but can be accessed using the enum
    EReturnOptions instead."""
    
    def __hash__(self):
        """:return: The WwiseObjectInfo hash."""
        return hash(self.guid)
    
    @classmethod
    def from_dict(cls, kvpairs: dict[str, _Any]) -> _Self:
        """
        Uses a dictionary to initialize a new instance.
        :param kvpairs: A dictionary to extract information from.
        :return: A new instance with id, name, type, and path populated. No additional properties.
        """
        guid = GUID(kvpairs["id"]) if kvpairs.get("id") is not None else GUID.get_null()
        name = Name(kvpairs["name"]) if kvpairs.get("name", "") != "" else Name.get_null()
        etype = EObjectType.from_type_name(kvpairs["type"]) if kvpairs.get("type") is not None else EObjectType.UNKNOWN
        path = ProjectPath(kvpairs["path"]) if kvpairs.get("path", "") != "" else ProjectPath.get_null()
        return cls(guid, name, etype, path)


@_dataclass
class TransportObjectInfo:
    """Data-only class storing information about a Wwise transport object."""
    
    object: GUID
    """The ID (GUID) of the object controlled by the transport object."""
    
    game_object: int
    """The game object used by the transport object. Unsigned integer 64-bit"""
    
    transport: int
    """Transport object ID. Unsigned Integer 32-bit."""
    
    def __hash__(self):
        """:return: The WwiseTransportObjectInfo hash."""
        return hash(self.object)
    
    @classmethod
    def from_dict(cls, kvpairs: dict[str, _Any]) -> _Self:
        """
        Uses a dictionary to initialize a new instance.
        :param kvpairs: A dictionary to extract information from.
        :return: A new instance with object GUID, game object, and transport object populated. No additional properties.
        """
        obj = GUID(kvpairs["object"]) if kvpairs.get("object") is not None else GUID.get_null()
        game_object = int(kvpairs["gameObject"]) if kvpairs.get("gameObject") is not None else -1
        transport = int(kvpairs["transport"]) if kvpairs.get("transport") is not None else -1
        return cls(obj, game_object, transport)


@_dataclass
class ContextMenuInfo:
    """Data-only class storing information about a command's context menu, which is part of an add-on command's
    arguments."""
    
    base_path: str = None
    """Defines a forward-separated path for the parent sub menus. If empty, the menu is inserted at the first level."""
    
    visible_for: set[EObjectType] = None
    """Defines a comma-separated list of the object types for which the item is visible. Refer to Wwise Objects
    Reference for the list of types supported. If empty, any type is allowed."""
    
    enabled_for: set[EObjectType] = None
    """Defines a comma-separated list of the object types for which the item is enabled. Refer to Wwise Objects
    Reference for the list of types supported. If empty, any type is allowed."""
    
    def __hash__(self):
        """:return: The ContextMenuInfo hash."""
        return hash(str(self.__dict__))
    
    @property
    def dictionary(self) -> dict[str, str]:
        as_dict = dict()
        if self.base_path is not None:
            as_dict["basePath"] = self.base_path
        if self.visible_for is not None:
            as_dict["visibleFor"] = ",".join([obj.get_type_name() for obj in self.visible_for])
        if self.enabled_for is not None:
            as_dict["enabledFor"] = ",".join([obj.get_type_name() for obj in self.enabled_for])
        return as_dict


@_dataclass
class MainMenuInfo:
    """Data-only class storing information about a command's main menu, which is part of an add-on command's argument"""
    
    main_menu_base_path: str
    """Defines a forward-separated path for the parent sub menus. It must at least define one level, which is associated
    to the top menu."""
    
    def __hash__(self):
        """:return: The ContextMenuInfo hash."""
        return hash(str(self.__dict__))
    
    @property
    def dictionary(self) -> dict[str, str]:
        as_dict = dict()
        as_dict["basePath"] = self.main_menu_base_path
        return as_dict


@_dataclass
class CommandInfo:
    """Data-only class storing information about an add-on command."""
    
    id: str
    """Defines a human readable unique ID for the command. To reduce risk of ID conflicts, please use a concatenation of
    the author name, the product name and the command name (e.g. 'mv.pywwise.do_something)."""
    
    display_name: str
    """Defines the name displayed in the user interface. (e.g. Do Something)"""
    
    program: str = None
    """Defines the program or script to run when the command is executed. Arguments are specified in 'args'. Note that
    common directories variables can be used, such as ${CurrentCommandDirectory}."""
    
    lua_script: str = None
    """Defines a lua script file path to run inside Wwise process when the command is executed. Arguments are specified
    in 'args'. Note that common directories variables can be used, such as ${CurrentCommandDirectory}."""
    
    lua_paths: list[str] = None
    """Defines an array of paths to be used to load additional lua scripts. Here is an example of a lua path
    C:/path_to_folder/?.lua. Note that common directories variables can be used, such as ${CurrentCommandDirectory}."""
    
    lua_selected_return: list[str] = None
    """Specifies an array of return expressions for the selected objects in Wwise. This will be available to the script
    in a lua table array in wa_args.selected. Several values provided for the option."""
    
    start_mode: EStartMode = EStartMode.SINGLE_SELECTION_SINGLE_PROCESS
    """Specifies how to expand variables in the arguments field in case of multiple selection in the Wwise user
    interface."""
    
    args: str = None
    """Defines the arguments. Refer to the documentation for the list of supported built-in variables. Note that in the
    event of a multiple selection, the variables are expanded based on the startMode field. Note that common directories
    variables can be used, such as ${CurrentCommandDirectory}."""
    
    cwd: str = None
    """Defines the current working directory to execute the program. Note that common directories variables can be used,
    such as ${CurrentCommandDirectory}."""
    
    default_shortcut: str = None
    """Defines the shortcut to use by default for this command. If the shortcut conflicts, it won't be used. This
    shortcut can be changed in the Keyboard Shortcut Manager."""
    
    redirect_outputs: bool = False
    """Defines if the standard output streams of the program (stdout + stderr) should be redirected and logged to Wwise
    on termination. The value is of boolean type and false by default."""
    
    context_menu: ContextMenuInfo = None
    """If present, specify how the command is added to Wwise context menus. If empty, no context menu is added."""
    
    main_menu: MainMenuInfo = None
    """If present, specify how the command is added to Wwise main menus. If empty, no main menu entry is added."""
    
    def __hash__(self):
        """:return: The CommandInfo ID hash."""
        return hash(self.id)
    
    @property
    def dictionary(self) -> dict[str, str | bool | ContextMenuInfo | MainMenuInfo]:
        as_dict = dict()
        as_dict["id"] = self.id
        as_dict["displayName"] = self.display_name
        as_dict["redirectOutputs"] = self.redirect_outputs
        as_dict["startMode"] = self.start_mode.value
        if self.program is not None:
            as_dict["program"] = self.program
        if self.lua_script is not None:
            as_dict["luaScript"] = self.lua_script
        if self.lua_paths is not None:
            as_dict["luaPaths"] = self.lua_paths
        if self.lua_selected_return is not None:
            as_dict["luaSelectedReturn"] = self.lua_selected_return
        if self.args is not None:
            as_dict["args"] = self.args
        if self.cwd is not None:
            as_dict["cwd"] = self.cwd
        if self.default_shortcut is not None:
            as_dict["defaultShortcut"] = self.default_shortcut
        if self.context_menu is not None:
            as_dict["contextMenu"] = self.context_menu.dictionary
        if self.main_menu is not None:
            as_dict["mainMenu"] = self.main_menu.dictionary
        return as_dict


@_dataclass
class SoundBankInfo:
    """A SoundBank's information."""
    
    name: str
    """The name of the SoundBank."""
    
    events: list[str] = None
    """List of events included in this SoundBank."""
    
    aux_busses: list[str] = None
    """List of AuxBus included in this SoundBank."""
    
    inclusions: list[str] = None
    """Inclusion type to use for this SoundBank."""
    
    rebuild: bool = False
    """Force rebuild of this particular SoundBank."""
    
    def __hash__(self) -> int:
        """:return: The SoundBankInfo hash."""
        return hash(self.name)
    
    @property
    def dictionary(self) -> dict[str, bool | str | list[str]]:
        """:return: The instance, represented as a dictionary."""
        as_dict = {"name": self.name, "rebuild": self.rebuild}
        if self.events is not None:
            as_dict["events"] = self.events
        if self.aux_busses is not None:
            as_dict["auxBusses"] = self.aux_busses
        if self.inclusions is not None:
            as_dict["inclusions"] = self.inclusions
        return as_dict


@_dataclass
class SoundBankInclusion:
    """Represents a SoundBank inclusion row."""
    
    obj: GUID | tuple[EObjectType, Name] | ProjectPath
    """The GUID, ProjectPath, or Name of the object to add/remove from the SoundBank's inclusion list.
    NOTE: Name is only supported for globally-unique names (e.g. Events, State Groups, etc.)."""
    
    filters: tuple[EInclusionFilter, ...]
    """Specifies what relations are being included. Possible Values: events, structures, media"""
    
    def __hash__(self) -> int:
        """:return: The instance, hashed."""
        return hash(self.obj)
    
    @property
    def dictionary(self) -> dict[str, list[str]]:
        """:return: The instance, represented as a dictionary."""
        as_dict = {
            "object": self.obj if not isinstance(self.obj, tuple) else f"{self.obj[0].get_type_name()}:{self.obj[1]}",
            "filter": list(set(self.filters))}
        return as_dict


@_dataclass
class LogItem:
    """A log item."""
    
    severity: ELogSeverity
    """The severity of the message."""
    
    time: int
    """Number of seconds elapsed since midnight (00:00:00), January 1, 1970, Coordinated Universal Time (UTC),
    according to the system clock."""
    
    id: str
    """The message ID for the log item."""
    
    description: str
    """The description of the log item."""
    
    @classmethod
    def from_dict(cls, kvpairs: dict[str, _Any]) -> _Self:
        """
        Uses a dictionary to initialize a new instance.
        :param kvpairs: A dictionary to extract information from.
        :return: A new instance with `severity`, `time`, `id`, and `description`.
        """
        severity = EnumStatics.from_value(ELogSeverity, kvpairs["severity"])
        time = kvpairs["time"]
        log_id = kvpairs["messageId"]
        description = kvpairs["message"]
        return cls(severity, time, log_id, description)


@_dataclass
class SwitchContainerAssignment:
    """Represents a switch container assigned (the relationship between a child object and a state/switch)."""
    
    child: GUID
    """The child object of a Switch Container, which is linked (assigned) to a State or Switch value."""
    
    state_or_switch: GUID
    """The State or Switch value to switch the child object is linked (assigned) to."""


@_dataclass
class CaptureLogItem:
    """A console log item."""
    
    type: ECaptureLogItemType
    """The type of the capture log item."""
    
    time: int
    """Number of seconds elapsed since midnight (00:00:00), January 1, 1970, Coordinated Universal Time (UTC),
    according to the system clock."""
    
    description: str
    """The description of the log item."""
    
    severity: ECaptureLogSeverity
    """The severity of the message."""
    
    wwise_object_id: GUID = GUID.get_null()
    """The GUID of the object for the entry."""
    
    wwise_object_name: Name = Name.get_null()
    """The name of the object for the entry."""
    
    wwise_object_short: ShortID = ShortID.get_null()
    """The short ID of the object for the entry."""
    
    game_object_id: GameObjectID = GameObjectID.get_null()
    """The game object ID for the entry."""
    
    game_object_name: Name = Name.get_null()
    """The game object name for the entry."""
    
    playing_id: PlayingID = PlayingID.get_null()
    """The playing ID for the entry."""
    
    error_code_name: str = ""
    """The error code name for the entry (e.g. `ErrorCode_VoiceStarting`)."""


@_dataclass
class SoundBankData:
    """A dataclass that represents a SoundBank (the actual data)."""
    
    b64data: str
    """Data of the SoundBank encoded in base64."""
    
    size: int
    """Size of the SoundBank data when decoded."""


@_dataclass
class PluginLibraryInfo:
    """A dataclass containing information about a plugin library."""
    
    name: str
    """The display name of the plugin."""
    
    short: ShortID
    """The short ID of the plugin."""
    
    type: str
    """A string describing the type of plugin."""
    
    dll: str
    """The name of the DLL for this plugin."""
    
    static: str
    """The name of the Static Lib for this plugin."""
    

@_dataclass
class SoundBankGenerationInfo:
    """Contains information about a SoundBank's generation."""
    
    guid: GUID
    """The GUID of the generated SoundBank object in Wwise."""
    
    name: Name
    """The short name of the SoundBank object in Wwise."""
    
    path: ProjectPath
    """The project path to the SoundBank object in Wwise."""
    
    file: str
    """The name of the BNK file - usually the Short ID with the **.bnk** extension at the end (e.g. 89627304.bnk)"""
    
    short: ShortID
    """The short ID of the SoundBank object in Wwise."""
    
    hash: GUID
    """Hash value (128-bit) written to the associated SoundBank .bnk file. Used during SoundBank generation to check
    existing .bnk files for changes."""
    
    type: EGeneratedSoundBankType
    """The type of SoundBank generated. Most banks are Auto-Defined SoundBanks, if your project has the feature
    enabled."""
    
    platform: Name
    """The name of the platform for which the SoundBank was generated."""
    
    language: Name
    """The name of the language for which the SoundBank was generated. Only present when generating a SoundBank for a
    specific language."""
    
    media: dict
    """JSON object containing information about Media."""
    
    external_sources: dict
    """JSON object containing information about External Sources."""
    
    plugins: dict
    """JSON object containing information about Plugins."""
    
    events: dict
    """JSON object containing information about Events."""
    
    dialogue_events: dict
    """JSON object containing information about Dialogue Events."""
    
    audio_busses: dict
    """JSON object containing information about Audio Busses."""
    
    aux_busses: dict
    """JSON object containing information about Aux Busses."""
    
    game_parameters: dict
    """JSON object containing information about Game Parameters."""
    
    triggers: dict
    """JSON object containing information about Triggers."""
    
    state_groups: dict
    """JSON object containing information about State Groups."""
    
    switch_groups: dict
    """JSON object containing information about Switch Groups."""
    
    acoustic_textures: dict
    """JSON object containing information about Acoustic Textures."""
    
    bank_data: SoundBankData = _field(default_factory=SoundBankData)
    """The bank data."""
    
    plugin_data: tuple[PluginLibraryInfo, ...] = _field(default_factory=tuple)
    """The plugin libs data."""
    
    error_message: str = ""
    """The error message, if an error occurred. Only present if an error occurred."""


@_dataclass
class AudioObjectMetadata:
    """ Contains information about the metadata associated with an audio object."""
    
    metadata_class_id: int
    """The class ID of the metadata. Unsigned Integer 32-bit. Range: [0,4294967295]."""
    
    source_short_id: ShortID = _field(default=ShortID.get_null())
    """The short ID of the source object. Unsigned Integer 32-bit. Range: [0,4294967295]."""
    
    metadata_name: Name = _field(default=Name.get_null())
    """The name of the metadata."""
    
    source_id: GUID = _field(default=GUID.get_null())
    """The ID (GUID) of the source object."""
    
    source_name: Name = _field(default=Name.get_null())
    """The name of the source object."""
    
    def __hash__(self):
        """:return: The AudioObjectMetadata hash."""
        return hash(self.source_short_id)


@_dataclass
class AudioObjectInfo:
    """Contains information about an audio object captured in the profiler."""
    
    audio_object_id: int
    """The ID of the Audio Object.Unsigned Integer 64-bit. Range: [0,18446744073709551615]."""
    
    bus_pipeline_id: int
    """The Pipeline ID of the Bus instance. Unsigned Integer 32-bit. Range: [0,4294967295]."""
    
    instigator_pipeline_id: int
    """The pipeline ID of the instigator from which the Audio Object originates. Can be either a Bus instance or a
    Voice. Unsigned Integer 32-bit. Range: [0,4294967295]."""
    
    effect_class_id: int
    """The Class ID of the effect after which the Audio Object was captured. Usage of AK_INVALID_UNIQUE_ID constant
    means that this Audio Object was captured before applying the first effect. Unsigned Integer 32-bit.
    Range: [0,4294967295]"""
    
    other: dict[EAudioObjectOptions | str, _Any] = _field(default_factory=dict)
    """A dictionary containing other information, if any. Keys are always strings, but can be accessed using the enum
    EAudioObjectOptions instead."""
    
    def __hash__(self):
        """:return: The AudioObject hash."""
        return hash(self.audio_object_id)


@_dataclass
class BusPipelineInfo:
    """Contains information about an audio bus captured in the profiler."""
    
    pipeline_id: int
    """Pipeline ID of the bus. Unsigned Integer 32-bit. Range: [0,4294967295]"""
    
    game_object_id: GameObjectID
    """Game Object ID corresponding to the voice. Unsigned Integer 64-bit. Range: [0,18446744073709551615]"""
    
    object_guid: GUID
    """Object GUID corresponding to the bus. An object GUID of the form: {aabbcc00-1122-3344-5566-77889900aabb}."""
    
    other: dict[EBusOptions | str, _Any] = _field(default_factory=dict)
    """A dictionary containing other information, if any. Keys are always strings, but can be accessed using the enum
    EBusOptions instead."""
    
    def __hash__(self):
        """:return: The BusPipelineInfo hash."""
        return hash(self.pipeline_id)


@_dataclass
class CPUStatisticsInfo:
    """Information about the amount of CPU percentage used by each element."""
    
    element_name: str
    """The name of the element on which we calculate CPU usage."""
    
    class_id: int
    """Class ID of the element."""
    
    instances: int
    """An estimation of the number of instances of the element."""
    
    type: str
    """The type of element. Examples: Codec, Source, Effect, Mixer, Sink, etc."""
    
    percent_inclusive: float
    """The percentage of CPU time spent in the execution of the element and those that it uses (calls)."""
    
    percent_exclusive: float
    """The percentage of CPU time spent only in the execution of the element itself."""
    
    milliseconds_inclusive: float
    """The milliseconds of CPU time spent in the execution of the element and those that it uses (calls)."""
    
    milliseconds_exclusive: float
    """The milliseconds of CPU time spent only in the execution of the element itself."""


@_dataclass
class GameObjectRegistrationData:
    """Data of a profiled game object and its registration data."""
    
    id: GameObjectID
    """The ID of the game object. Unsigned Integer 64-bit. Range: [0,18446744073709551615]."""
    
    name: str
    """The name of the game object."""
    
    registration_time: int
    """The time at which the game object was registered. Integer 32-bit. Range: [-2147483648,2147483647]."""
    
    unregistration_time: int
    """The time at which the game object was unregistered. Integer 32-bit. Range: [-2147483648,2147483647]."""


@_dataclass
class LoadedMediaInfo:
    """Information about a media file loaded into memory as a result of the PrepareEvent() and PrepareGameSyncs()
    functions."""
    
    media_id: ShortID
    """The short ID of the media file."""
    
    file_name: Name
    """The name of the media file."""
    
    audio_format: str
    """The audio format of the media file."""
    
    size: int
    """The size (in bytes) of the media file."""
    
    sound_bank: Name
    """The name of the SoundBank that contains the media file."""


@_dataclass
class PerformanceMonitorCounterInfo:
    """Information about a performance monitor counter and its value."""
    
    name: str
    """Name of the counter as shown in Wwise Authoring."""
    
    identifier: str
    """Unique identifier of the counter."""
    
    value: float
    """Value of counter at given time. If invalid, value will be equivalent to `float("-inf")`."""


@_dataclass
class ActiveRTPCInfo:
    """Information about an active RTCP associated with a playing voice."""
    
    guid: GUID
    """The ID (GUID) of the Game Parameter, LFO, Time, Envelope or MIDI Parameter object. An object GUID of the form:
    {aabbcc00-1122-3344-5566-77889900aabb}."""
    
    name: Name
    """The name of the Game Parameter, LFO, Time, Envelope or MIDI Parameter object. The name of the object."""
    
    game_object_id: GameObjectID
    """The Game Object associated with the RTPC scope, or AK_INVALID_GAME_OBJECT for global scope RTPCs. A game object
    ID, unsigned integer 64-bit. Range: [0,18446744073709551615]."""
    
    value: float
    """The value of the Game Parameter, LFO, Time, Envelope or MIDI Parameter at the cursor time. If invalid, the value
    will be equivalent to `float("-inf")`."""


@_dataclass
class StreamObjectInfo:
    """Data-only class storing information about how each of the streams is managed by the Wwise sound engine."""
    
    device_name: Name
    """The name of the device from which the stream emanates."""
    
    stream_name: Name
    """The name given to the stream."""
    
    file_size: int
    """The size of the file being streamed."""
    
    file_position: float
    """The position of the stream within the file, given as a percentage."""
    
    priority: int
    """The priority of the stream."""
    
    bandwidth_total: int
    """The rate at which the file was streamed in the last profiling frame. This value takes all transfers into account,
    including transfers that occurred from the Stream Manager's cache."""
    
    bandwidth_low_level: int
    """The rate at which the file was streamed in the last profiling frame. Unlike the Total Bandwidth field, this field
    value considers transfers that occurred from within the low-level device."""
    
    referenced_memory: int
    """The amount of memory that is referenced by the stream. This excludes memory used for I/O transfers. It can be
    seen as a measure of how much data the stream may grant to the sound engine at any given time."""
    
    estimated_throughput: int
    """The estimated throughput of the stream. The sound engine estimates the rate at which it consumes data from a
    stream according to its encoding format and number of channels."""
    
    active: bool
    """Indicates True if the stream was active at least once during the last profiling frame."""
    
    target_buffer_size: int
    """The streaming device's target buffer length."""
    
    buffer_status_buffered: float
    """The portion of requested data that is buffered, given as a percentage of the target buffer size."""
    
    def __hash__(self):
        """:return: The WwiseObjectInfo hash."""
        return hash(self.device_name)
    
    @property
    def dictionary(self) -> dict[str, str | bool | int | Name]:
        """:return: The instance represented as a dictionary."""
        as_dict = dict()
        if self.device_name is not None:
            as_dict["deviceName"] = self.device_name
        if self.stream_name is not None:
            as_dict["streamName"] = self.stream_name
        if self.file_size is not None:
            as_dict["fileSize"] = self.file_size
        if self.file_position is not None:
            as_dict["filePosition"] = self.file_position
        if self.priority is not None:
            as_dict["priority"] = self.priority
        if self.bandwidth_total is not None:
            as_dict["bandwidthTotal"] = self.bandwidth_total
        if self.bandwidth_low_level is not None:
            as_dict["bandwidthLowLevel"] = self.bandwidth_low_level
        if self.referenced_memory is not None:
            as_dict["referencedMemory"] = self.referenced_memory
        if self.estimated_throughput is not None:
            as_dict["estimatedThroughput"] = self.estimated_throughput
        if self.target_buffer_size is not None:
            as_dict["targetBufferSize"] = self.target_buffer_size
        if self.buffer_status_buffered is not None:
            as_dict["bufferStatusBuffered"] = self.buffer_status_buffered
        return as_dict


@_dataclass
class VoiceContributionParameter:
    """Data class containing the information relating to contribution parameters associated to the voice inspector
    object."""
    
    property_type: str
    """The object property affecting the voice."""
    
    reason: str
    """The reason for the parameter to affect the voice."""
    
    driver: GUID | str
    """Can either be: The driving object GUID or the driving reason when a parameter is not driven by an object."""
    
    driver_value: float | GUID
    """Can either be: The value of the driver affecting the parameter or an object GUID."""
    
    value: float
    """Contribution value."""
    
    def __hash__(self):
        """:return: The VoiceInspectorContributionObjectProperties hash."""
        return hash(str(self.__dict__))
    
    @property
    def dictionary(self) -> dict[str, float | str | GUID]:
        """:return: The instance represented as a dictionary."""
        as_dict = dict()
        as_dict["propertyType"] = self.property_type
        as_dict["reason"] = self.reason
        as_dict["driver"] = self.driver
        as_dict["driverValue"] = self.driver_value
        as_dict["value"] = self.value
        return as_dict


@_dataclass
class VoiceInspectorContribution:
    """Data class containing information from a single voice inspector contribution object."""
    
    name: str
    """The name of the contribution."""
    
    volume: float
    """The volume difference applied."""
    
    lpf: float
    """The LPF difference applied."""
    
    hpf: float
    """The HPF difference applied."""
    
    parameters: list[VoiceContributionParameter] = _field(default_factory=list)
    """Contribution parameters associated to the object."""
    
    children: list[_Self] = _field(default_factory=list)
    """An array of child voice contribution objects associated to the object."""
    
    def __hash__(self):
        """:return: The instance's hash."""
        return hash(str(self.__dict__))
    
    @property
    def dictionary(self) -> dict[str, str | float | dict]:
        """:return: The instance represented as a dictionary."""
        as_dict = dict()
        as_dict["name"] = self.name
        as_dict["volume"] = self.volume
        as_dict["lpf"] = self.lpf
        as_dict["hpf"] = self.hpf
        as_dict["children"] = self.children
        as_dict["parameters"] = self.parameters
        return as_dict


@_dataclass
class VoiceContributionHierarchy:
    """Data class containing relevant to the return schema used to store the information of all parameters affecting
    a voice pipeline ID."""
    
    volume: float
    """The volume difference applied as a contribution."""
    
    lpf: float
    """The LPF difference applied as a contribution."""
    
    hpf: float
    """The HPF difference applied as a contribution."""
    
    objects: list[VoiceInspectorContribution] = _field(default_factory=list)
    """A dictionary of `VoiceInspectorContribution` objects."""
    
    def __hash__(self):
        """:return: The instance's hash."""
        return hash(str(self.__dict__))
    
    @property
    def dictionary(self) -> dict[str, str | float | dict]:
        """:return: The instance represented as a dictionary."""
        as_dict = dict()
        as_dict["volume"] = self.volume
        as_dict["lpf"] = self.lpf
        as_dict["hpf"] = self.hpf
        as_dict["objects"] = self.objects
        return as_dict


@_dataclass
class PlayingVoiceProperties:
    """Data class containing information about the properties of a playing voice."""
    
    pipeline_id: int
    """Pipeline ID of the voice."""
    
    game_object_id: GameObjectID
    """Game Object ID corresponding to the voice."""
    
    object_guid: GUID
    """Object GUID corresponding to the voice."""
    
    other: dict[EVoicePipelineReturnOptions | str, _Any] = _field(default_factory=dict)
    """A dictionary containing other information, if any. Keys are always strings, but can be accessed using the enum
    EVoicePipelineReturnOptions instead."""
    
    def __hash__(self):
        """:return: The WwiseObjectInfo hash."""
        return hash(self.object_guid)


class WwiseObjectWatch:
    """Represents a watch. Used for setting up the `ak.wwise.core.object.property_changed` event. For the specified
    GUID, changes to any of the specified properties will trigger the event."""
    
    guid: GUID
    """The GUID of the object to watch."""
    
    properties: tuple[str]
    """A collection of properties names to watch."""


@_dataclass
class SourceControlStatus:
    """Represents a file's source control status."""
    
    status: str
    """A description of the status."""
    
    owner: str
    """The name of the file's owner."""


@_dataclass
class SourceFileInfo:
    """Stores source control and project information about a source file."""
    
    path: OriginalsPath
    """The absolute path of the source file."""
    
    usage: tuple[WwiseObjectInfo, ...]
    """The Wwise objects that use the source file."""
    
    is_missing: bool
    """Indicates if the file is absent in the source manager."""
    
    @property
    def is_used(self) -> bool:
        """
        Whether the source file is currently used in the project.
        :return: `True` if `usage` is not empty; else, `False`.
        """
        return bool(self.usage)


@_dataclass
class GraphPoint2D:
    """A dataclass describing a point on a 2D graph."""
    
    position: Vector2
    """The position of the point on the graph."""
    
    shape: EAttenuationCurveShape = EAttenuationCurveShape.LINEAR
    """The shape formed by the point on the graph."""


@_dataclass
class AttenuationCurve:
    """A dataclass representing an attenuation curve (e.g. a Low-Pass Filter in an attenuation shareset)."""
    
    points: tuple[GraphPoint2D, ...]
    """A collection of points on the graph."""
    
    usage: EAttenuationCurveUsage
    """The attenuation curve usage."""
    
    etype: EAttenuationCurveType
    """The type of the attenuation curve."""


@_dataclass
class PropertyInfo:
    """A dataclass representing information about a property."""
    
    name: str
    """The name of the property."""
    
    display_name: str
    """The property's display name, as used in the Wwise UI."""
    
    audio_engine_id: int
    """The audio engine ID of the property."""
    
    default: _Any
    """The property's default value."""
    
    stype: str
    """The name of the property's data type."""
    
    rtpc_mode: ERtpcMode
    """The rtpc mode supported by the property."""
    
    supports_unlinking: bool | None
    """Whether the property supports unlinking. If this information is unknown, the value is `None` instead."""
    
    supports_randomizer: bool | None
    """Whether the property supports randomizers. If this information is unknown, the value is `None` instead."""


@_dataclass
class WwiseVersionInfo:
    """A dataclass representing a Wwise version."""
    
    display_name: str
    """Wwise version name."""
    
    year: int
    """Version year. Range: [2000,2100]."""
    
    major: int
    """Version's major number Range: [0,100]."""
    
    minor: int
    """Version's minor number. Range: [0,100]."""
    
    build: int
    """Build number. Range: [1,*]."""
    
    nickname: str
    """Special name given to a version."""
    
    schema: int
    """Schema version for the Wwise Project and Work Units. Range: [1,*]."""


@_dataclass
class WwiseDirectories:
    """A dataclass representing the collection of directories used by Wwise."""
    
    install: SystemPath
    """The root directory of Wwise. This is the installation directory."""
    
    authoring: SystemPath
    """The Wwise Authoring root directory."""
    
    bin: SystemPath
    """The bin directory, where Wwise.exe is located."""
    
    help: SystemPath
    """The help directory."""
    
    user: SystemPath
    """The Wwise user data directory root."""


@_dataclass
class RemoteConsoleInformation:
    """A dataclass representing the information of an available console capable of connecting Wwise authoring to
    a Sound Engine instance."""
    
    name: str
    """Name of the remote console as returned by the executable."""
    
    platform: str
    """Platform of the remote console as returned by the executable."""
    
    custom_platform: str
    """Platform, as defined in the project platforms of the remote console as returned by the executable."""
    
    host: str
    """The IPv4 of the connected host. This can also be a file path if Wwise is connected to a local file
    (profiler session). For using WAAPI on Mac, please refer to Using WAAPI on Mac."""
    
    app_name: str
    """The name of the connected application as returned by the executable. Must be used when connecting to a specific
     Sound Engine instance."""
    
    command_port: int
    """The command port. Can be used when connecting to a specific Sound Engine instance."""
    
    def __hash__(self):
        """:return: The instance's hash."""
        return hash(str(self.__dict__))
    
    @property
    def dictionary(self) -> dict[str, str | int]:
        """:return: The instance represented as a dictionary."""
        as_dict = dict()
        as_dict["name"] = self.name
        as_dict["platform"] = self.platform
        as_dict["customPlatform"] = self.custom_platform
        as_dict["host"] = self.host
        as_dict["appName"] = self.app_name
        as_dict["commandPort"] = self.command_port
        return as_dict


@_dataclass
class ConnectionStatusInfo:
    """A dataclass containing information about the current connection."""
    
    is_connected: bool
    """Indicates if the Wwise Authoring application is connected to a Wwise Sound Engine process."""
    
    status: str
    """The current connection status in text."""
    
    console: RemoteConsoleInformation
    """Remote console associated with the connection."""
    
    def __hash__(self):
        """:return: The instance's hash."""
        return hash(str(self.__dict__))
    
    @property
    def dictionary(self) -> dict[str, str | int]:
        """:return: The instance represented as a dictionary."""
        as_dict = dict()
        as_dict["isConnected"] = self.is_connected
        as_dict["status"] = self.status
        as_dict["console"] = self.console
        return as_dict


@_dataclass
class AudioImportEntry:
    """Dataclass representing a single entry for an audio import operation."""
    
    object_path: ProjectPath
    """The project path of the Wwise object to create, including the name of the object to create. Example:
    "/Actor-Mixer Hierarchy/Default Work Unit/<Random Container>MyContainer/<Sound SFX>MySound" will create a
    `RandomSequenceContainer` (with `RandomOrSequence=1`) called "MyContainer" and a `Sound` called "MySound"."""
    
    root_path: GUID | tuple[EObjectType, Name] | ProjectPath = None
    """Object ID (GUID), name, or path used as root relative object paths. The name of the object qualified by its type
    or Short ID in the form of type:name or Global:shortId. Only object types that have globally-unique names or Short
    Ids are supported. Ex: Event:Play_Sound_01, Global:245489792 string A project path to a Wwise object, including the
    category and the work-unit. For example: /Actor-Mixer Hierarchy/Default Work Unit/New Sound SFX."""
    
    audio_file_path: SystemPath = None
    """Path to media file to import. This path must be accessible from Wwise. For using WAAPI on Mac, please refer to
    Using WAAPI on Mac ."""
    
    audio_file_base64: bytes = None
    """Base64 encoded WAV audio file data to import with its target file path relative to the Originals folder,
    separated by a vertical bar. E.g. 'MySound.wav|UklGRu...'."""
    
    originals_path: OriginalsPath = None
    """Specifies the 'originals' sub-folder in which to place the imported audio file. This folder is relative to the
    'originals' folder in which the file would normally be imported. Example: if importing an SFX, the audio file
    is imported to the folder "/Originals/SFX/{originals_path}"."""
    
    object_type: EObjectType = None
    """Specifies the type of object to create when importing an audio file. This type can also be specified directly in
    the objectPath. Refer to Wwise Objects Reference for the available types."""
    
    object_notes: str = ""
    """The "Notes" field of the created object."""
    
    source_notes: str = ""
    """The "Notes" field of the created audio source object."""
    
    event: ProjectPath = None
    """Defines the path and name of an Event to be created for the imported object.
    Example: "/Events/Default Work Unit/MyEvent"."""
    
    dialogue_event: ProjectPath = None
    """Defines the path and name of a Dialogue Event to be created for the imported object.
    Example: "/Dynamic Dialogue/Default Work Unit/MyEvent"."""
    
    language: GUID | Name = None
    """Imports the language for the audio file import (taken from the project's defined languages, found in the WPROJ
    file LanguageList)."""
    
    properties: ListOrTuple[tuple[str, _NoneType | bool | int | float | str]] = ()
    """A collection of key-value pairs, where keys are property names prefixed by either `@` (a reference to the
    associated object) or `@@` (a reference to the source of override)."""
    
    @property
    def dictionary(self) -> dict[str, str | int | float | bool | _Any | None]:
        """:return: The instance represented as a dictionary"""
        return {**({"objectPath": self.object_path} if self.object_path is not None else {}),
                **({"importLocation": self.root_path} if self.root_path is not None else {}),
                **({"audioFile": str(self.audio_file_path)} if self.audio_file_path is not None else {}),
                **({"audioFileBase64": self.audio_file_base64} if self.audio_file_base64 is not None else {}),
                **({"originalsSubFolder": self.originals_path} if self.originals_path is not None else {}),
                **({"objectType": self.object_type.get_type_name()} if self.object_type is not None else {}),
                **({"notes": self.object_notes} if self.object_notes is not None else {}),
                **({"audioSourceNotes": self.source_notes} if self.source_notes is not None else {}),
                **({"event": self.event} if self.event is not None else {}),
                **({"dialogueEvent": self.dialogue_event} if self.dialogue_event is not None else {}),
                **({"importLanguage": self.language} if self.language is not None else {}),
                **dict(self.properties)}


@_dataclass
class ConversionLogItem:
    """Dataclass representing a log item related to conversions."""
    
    severity: ELogSeverity
    """Severity of the logged message."""
    
    message: str
    """Message logged by the system."""


@_dataclass
class WwiseGlobalDirectories:
    """A dataclass containing paths to standard Wwise directories."""
    
    install: SystemPath
    """The root directory of Wwise. This is the installation directory."""
    
    authoring: SystemPath
    """The Wwise Authoring root directory."""
    
    binaries: SystemPath
    """The bin directory, where Wwise.exe is located."""
    
    helper: SystemPath
    """The help directory."""
    
    user: SystemPath
    """The Wwise user data directory root."""


@_dataclass
class WwiseGlobalInfo:
    """Global information about Wwise."""
    
    session_id: GUID
    """Wwise session ID."""
    
    api_version: float
    """Version of the Wwise Authoring API. Range: [1, *]"""
    
    display_name: str
    """Wwise display name."""
    
    branch: str
    """Branch built."""
    
    version: WwiseVersionInfo
    """Wwise version information."""
    
    configuration: EWwiseBuildConfiguration
    """Indicates the build configuration type (e.g. release)."""
    
    platform: EWwiseBuildPlatform
    """Indicates the platform on which Wwise was built (e.g. win32)."""
    
    is_command_line: bool
    """Whether Wwise is running in command line."""
    
    process_id: int
    """The process identifier of Wwise."""
    
    process_path: str
    """The process path of Wwise."""
    
    directories: WwiseGlobalDirectories
    """Collection or directories used by Wwise."""
    
    copyright_info: str
    """Copyright information."""


@_dataclass
class LanguageInfo:
    """Language information."""
    
    guid: GUID
    """Language identifier."""
    
    name: Name
    """Language name."""
    
    short_id: ShortID
    """Language short (32-bit) identifier."""


@_dataclass
class WwiseProjectDirectories:
    """Dataclass with information about the Wwise project directories."""
    
    root: SystemPath
    """The root directory of the Wwise project."""
    
    cache: SystemPath
    """The cache directory of the Wwise project, as specified in the Project Settings. Contains generated media files
    in the WEM format."""
    
    originals: SystemPath
    """The Originals directory of the Wwise project, as specified in the Project Settings. Contains original media files
    in the WAV format."""
    
    sound_bank_output_root: SystemPath
    """The SoundBank output root directory of the project, as specified in the Project Settings. This is where BNK, TXT
    H, XML, and JSON files are placed after a SoundBank generation."""
    
    commands: SystemPath
    """The Commands directory of the project, as specified in the Project Settings. This is where JSON files that
    specify add-on commands are stored."""
    
    properties: SystemPath
    """The Properties directory of the project. This is usually the root Wwise project directory."""


@_dataclass
class WwiseProjectInfo:
    """Information about a Wwise project."""
    
    name: Name
    """The name of the Wwise project as stored in the XML data in the WPROJ file. This is NOT the name of the WPROJ
    file."""
    
    title: str
    """The complete text from the Wwise titlebar."""
    
    path: SystemPath
    """The absolute path of the WPROJ file."""
    
    guid: GUID
    """Project identifier."""
    
    is_dirty: bool
    """True if the Project or any of the Work Units have unsaved changes."""
    
    current_language_guid: GUID
    """The current Language set in the user interface."""
    
    reference_language_guid: GUID
    """The reference Language set in the Language settings."""
    
    current_platform_guid: GUID
    """The current Platform set in the user interface."""
    
    languages: tuple[LanguageInfo, ...]
    """The languages defined in the project."""
    
    platforms: tuple[PlatformInfo, ...]
    """The platforms defined in the project."""
    
    default_conversion: WwiseObjectInfo
    """The default Conversion Settings object."""


@_dataclass
class SetImportNode:
    """A node defining an audio import. Can be for SFX or Voice assets. This class is intended to be used in combination
    with **SetObjectNode** and **SetOperation**."""
    
    audio_file_path: SystemPath
    """Path to media file to import. This path must be accessible from Wwise."""
    
    originals_path: OriginalsPath = None
    """The path to the subfolder where to import the files. In this case, **omit** the "SFX" and "Voice" default
    folders. Although not required, specifying a subfolder here is highly recommended."""
    
    language: Name = None
    """If importing a voice asset, specify the name of the target language."""
    
    def dict(self):
        """
        Convert this data structure to a JSON-like dictionary.
        :return: The dictionary, in a JSON-like format.
        """
        return {"audioFile": str(self.audio_file_path),
                **({"originalsSubFolder": str(self.audio_file_path)} if self.audio_file_path is not None else {}),
                **({"language": self.language} if self.language is not None else {})}


@_dataclass
class SetObjectNode:
    """A node defining the creation or modification of an object. This is intended to be used as the data type for
    children nodes in **ObjectSetOperation**, which serves as a root node."""
    
    type: EObjectType
    """The type of the new object to create. Not all types are supported in all cases. Refer to the Wwise documentation
    if unsure."""
    
    name: str | Name
    """The name of the new object to create."""
    
    properties: ListOrTuple[tuple[WwiseProperty | str, _Any]] = _field(default_factory=tuple)
    """The properties to set. This is a **list** or **tuple** of property-value pairs (a **tuple** of size 2).
    **Example: [(Sound.position_3d, E3DPosition.EMITTER), (Sound.volume, -6.0)]**"""
    
    children: ListOrTuple[_Self] = _field(default_factory=tuple)
    """The children objects to create. Setting properties is possible too."""
    
    audio_imports: ListOrTuple[SetImportNode] = None
    """The audio assets to import. This also created Sound objects (SFX or Voice)."""
    
    notes: str = ""
    """The notes or comments for this objects, which are displayed in the "Notes" box in Wwise."""
    
    def dict(self) -> dict:
        """
        Convert this data structure to a JSON-like dictionary.
        :return: The dictionary, in a JSON-like format.
        """
        _dict = {"type": self.type.get_type_name(),
                 "name": self.name,
                 "notes": self.notes,
                 "children": [child.dict() for child in self.children] if self.children else None,
                 "import": {"files": [file.dict() for file in self.audio_imports]} if self.audio_imports else None}
        for prop in self.properties:
            _dict[f"@{prop[0]}"] = prop[1]
        return {k: v for k, v in _dict.items() if v is not None and (k != "import" or v.get("files", None))}


@_dataclass
class SetOperation:
    """A structure used for setting and/or modifying objects in batches. Behaves like a unidirectional linked-list,
    where the instance of this class is the root of the list."""
    
    root: GUID | ProjectPath
    """The GUID or project path of the existing object on which to set the name, notes, properties or references and
    under which to create children or create objects in a list. If using a path, use the format: **'/Actor-Mixer
    Hierarchy/Default Work Unit/New Sound SFX'**.'"""
    
    children: ListOrTuple[SetObjectNode] = None
    """The children objects to create. Setting properties is possible too."""
    
    audio_imports: ListOrTuple[AudioImportEntry] = None
    """The audio assets to import. This also created Sound objects (SFX or Voice)."""
    
    def dict(self) -> dict:
        """
        Convert this data structure to a JSON-like dictionary.
        :return: The dictionary, in a JSON-like format.
        """
        _dict = {"object": self.root,
                 "children": [child.dict() for child in self.children] if self.children else None,
                 "import": {"files": [file.dict() for file in self.audio_imports]} if self.audio_imports else None}
        return {k: v for k, v in _dict.items() if v is not None}



# Copyright 2025 Matheus Vilano
# SPDX-License-Identifier: Apache-2.0

from pywwise.descriptors import WwiseProperty
from pywwise.enums import EColour, EMusicPlaylistItemType, EPlaylistMode, ERandomType
from pywwise.objects.abc import WwiseObject
from pywwise.objects.types.music_segment import MusicSegment


class MusicPlaylistItem(WwiseObject):
    """
    https://www.audiokinetic.com/en/library/edge/?source=SDK&id=wwiseobject_musicplaylistitem.html \n
    A class serving as an interface for getting/setting properties on Wwise objects. This type specifically targets
    the class represented by `EObjectType.MUSIC_PLAYLIST_ITEM`.
    """
    colour = WwiseProperty[EColour]("Color", EColour)
    loop_count = WwiseProperty[int]("LoopCount", int)
    normal_or_shuffle = WwiseProperty[ERandomType]("NormalOrShuffle", ERandomType)
    override_colour = WwiseProperty[bool]("OverrideColor", bool)
    play_mode = WwiseProperty[EPlaylistMode]("PlayMode", EPlaylistMode)
    playlist_item_type = WwiseProperty[EMusicPlaylistItemType]("PlaylistItemType", EMusicPlaylistItemType)
    random_avoid_repeating_count = WwiseProperty[int]("RandomAvoidRepeatingCount", int)
    segment = WwiseProperty[MusicSegment]("Segment", MusicSegment)
    weight = WwiseProperty[float]("Weight", float)



# Copyright 2024 Matheus Vilano
# SPDX-License-Identifier: Apache-2.0

from waapi import WaapiClient as _WaapiClient

from pywwise.aliases import SystemPath
from pywwise.structs import ConnectionStatusInfo, RemoteConsoleInformation


class Remote:
    """ak.wwise.core.remote"""
    
    def __init__(self, client: _WaapiClient):
        """
        Constructor.
        :param client: The WAAPI client to use.
        """
        self._client = client
    
    def connect(self, host: str | SystemPath, app_name: str = None, command_port: int = None) -> bool:
        """
        https://www.audiokinetic.com/library/edge/?source=SDK&id=ak_wwise_core_remote_connect.html \n
        Connects the Wwise Authoring application to a Wwise Sound Engine running executable or to a saved
        profile file. The host must be running code with communication enabled. If only "host" is
        provided, Wwise connects to the first Sound Engine instance found. To distinguish between
        different instances, you can also provide the name of the application to connect to.
        :param host: The host to connect to.The host can be a computer name, an IPv4 address, an IP:PORT pair, or a
                     full path to a saved capture (.prof file). Use 127.0.0.1 to connect to localhost.
        :param app_name: The value in the Application Name column from the Remote Connection dialog in Wwise, or from
                         the get_available_consoles() method. If you are running more than one Sound Engine instance,
                         you can specify the name of the application to connect to.
        :param command_port: The command port. If you are running two or more Sound Engine instances that use the same
                             application name, you can specify the command port to distinguish between different
                             applications sharing the same name. You don't need to use this if the application name
                             is unique. When using this, you must also provide appName. You can obtain this information
                             from the get_available_consoles() method.
        :return: True if the connection is successful, False otherwise.
        """
        args = {"host": host}
        
        if app_name is not None:
            args["appName"] = app_name
        
        if command_port is not None and app_name is None:
            return False
        else:
            args["appName"] = app_name
            args["commandPort"] = command_port
        
        return self._client.call("ak.wwise.core.remote.connect", args) is not None
    
    def disconnect(self) -> bool:
        """
        https://www.audiokinetic.com/library/edge/?source=SDK&id=ak_wwise_core_remote_disconnect.html \n
        Disconnects the Wwise Authoring application from a connected Wwise Sound Engine running executable.
        :return: True if the disconnection is successful, False otherwise.
        """
        return self._client.call("ak.wwise.core.remote.disconnect", {}) is not None
    
    def get_available_consoles(self) -> tuple[RemoteConsoleInformation, ...]:
        """
        https://www.audiokinetic.com/library/edge/?source=SDK&id=ak_wwise_core_remote_getavailableconsoles.html \n
        Retrieves all consoles available for connecting Wwise Authoring to a Sound Engine instance.
        :return: A tuple of RemoteConsoleInformation objects. Each object represents an available remote console.
        """
        results = self._client.call("ak.wwise.core.remote.getAvailableConsoles", {})
        
        if results is None:
            return tuple()
        
        results = results.get("consoles")
        
        if results is None:
            return tuple()
        
        consoles = [RemoteConsoleInformation(
            name=console.get("name", ""),
            platform=console.get("platform", ""),
            custom_platform=console.get("customPlatform", ""),
            host=console.get("host", ""),
            app_name=console.get("appName", ""),
            command_port=console.get("commandPort", -1),
        ) for console in results]
        
        return tuple(consoles)
    
    def get_connection_status(self) -> ConnectionStatusInfo:
        """
        https://www.audiokinetic.com/library/edge/?source=SDK&id=ak_wwise_core_remote_getconnectionstatus.html \n
        Retrieves the connection status.
        :return: A ConnectionStatusInfo object, detailing into more detail the state of current connection.
        """
        result = self._client.call("ak.wwise.core.remote.getConnectionStatus", {})
        
        if result is None:
            result = dict()
        
        console = result.get("console", dict())
        console = RemoteConsoleInformation(
            name=console.get("name", ""),
            platform=console.get("platform", ""),
            custom_platform=console.get("customPlatform", ""),
            host=console.get("host", ""),
            app_name=console.get("appName", ""),
            command_port=console.get("commandPort", -1))
        
        return ConnectionStatusInfo(is_connected=result.get("isConnected", False),
                                    status=result.get("status", "Failed to retrieve connection status."),
                                    console=console)



# Copyright 2025 Matheus Vilano
# SPDX-License-Identifier: Apache-2.0

from pywwise.descriptors import WwiseProperty
from pywwise.enums import EStereoDelayFilterType, EStereoDelayInput
from pywwise.objects.abc import WwiseObject


class WwiseStereoDelay(WwiseObject):
    """
    https://www.audiokinetic.com/en/library/edge/?source=SDK&id=wwiseobject_effect_wwise_stereo_delay.html \n
    A class serving as an interface for getting/setting properties on Wwise objects. This type specifically targets
    the class represented by `EObjectType.WWISE_STEREO_DELAY`.
    """
    dry_level = WwiseProperty[float]("DryLevel", float)
    enable_cross_feed = WwiseProperty[bool]("EnableCrossFeed", bool)
    enable_feedback = WwiseProperty[bool]("EnableFeedback", bool)
    filter_frequency = WwiseProperty[float]("FilterFrequency", float)
    filter_gain = WwiseProperty[float]("FilterGain", float)
    filter_q = WwiseProperty[float]("FilterQFactor", float)
    filter_type = WwiseProperty[EStereoDelayFilterType]("FilterType", EStereoDelayFilterType)
    front_rear_balance = WwiseProperty[float]("FrontRearBalance", float)
    left_crossfeed = WwiseProperty[float]("LeftCrossfeed", float)
    left_delay_time = WwiseProperty[float]("LeftDelayTime", float)
    left_feedback = WwiseProperty[float]("LeftFeedback", float)
    left_input_type = WwiseProperty[EStereoDelayInput]("LeftInputType", EStereoDelayInput)
    right_crossfeed = WwiseProperty[float]("RightCrossfeed", float)
    right_delay_time = WwiseProperty[float]("RightDelayTime", float)
    right_feedback = WwiseProperty[float]("RightFeedback", float)
    right_input_type = WwiseProperty[EStereoDelayInput]("RightInputType", EStereoDelayInput)
    wet_level = WwiseProperty[float]("WetLevel", float)



# Copyright 2025 Matheus Vilano
# SPDX-License-Identifier: Apache-2.0

from pywwise.descriptors import WwiseProperty
from pywwise.enums import (EConvolutionReverbAlgorithm, EConvolutionReverbBlockSize,
                           EConvolutionReverbIrChannelSelection, EConvolutionReverbIrLpfSlope, ESpeakerBitMask)
from pywwise.objects.abc import WwiseObject


class WwiseConvolutionReverb(WwiseObject):
    """
    https://www.audiokinetic.com/en/library/edge/?source=SDK&id=wwiseobject_effect_wwise_convolution_reverb.html \n
    A class serving as an interface for getting/setting properties on Wwise objects. This type specifically targets
    the class represented by `EObjectType.WWISE_CONVOLUTION_REVERB`.
    """
    algo_type_select = WwiseProperty[EConvolutionReverbAlgorithm]("AlgoTypeSelect", EConvolutionReverbAlgorithm)
    centre_level = WwiseProperty[float]("CenterLevel", float)
    channel_config_override = WwiseProperty[ESpeakerBitMask]("ChannelConfigOverride", ESpeakerBitMask)
    dry_level = WwiseProperty[float]("DryLevel", float)
    front_level = WwiseProperty[float]("FrontLevel", float)
    front_rear_delay = WwiseProperty[float]("FrontRearDelay", float)
    full_precision_media = WwiseProperty[bool]("FullPrecisionMedia", bool)
    hardware_acceleration = WwiseProperty[bool]("HardwareAcceleration", bool)
    ir_channel_selection = WwiseProperty[EConvolutionReverbIrChannelSelection](
        "IRChannelSelect", EConvolutionReverbIrChannelSelection)
    ir_convolution_smooth = WwiseProperty[float]("IRConvolutionSmooth", float)
    ir_convolution_smooth_max = WwiseProperty[float]("IRConvolutionSmoothMax", float)
    ir_convolution_start = WwiseProperty[float]("IRConvolutionStart", float)
    ir_convolution_stop = WwiseProperty[float]("IRConvolutionStop", float)
    ir_convolution_threshold = WwiseProperty[float]("IRConvolutionThreshold", float)
    ir_graphic_eq_automate = WwiseProperty[bool]("IRGraphicEQAutomate", bool)
    ir_lpf_automate = WwiseProperty[bool]("IRLPFAutomate", bool)
    ir_lpf_slope = WwiseProperty[EConvolutionReverbIrLpfSlope]("IRLPFSlope", EConvolutionReverbIrLpfSlope)
    ir_lr_mix = WwiseProperty[float]("IRLRMix", float)
    ir_level = WwiseProperty[float]("IRLevel", float)
    ir_level_automate = WwiseProperty[bool]("IRLevelAutomate", bool)
    ir_stretch = WwiseProperty[float]("IRStretch", float)
    input_centre_level = WwiseProperty[float]("InputCenterLevel", float)
    input_lfe_level = WwiseProperty[float]("InputLFELevel", float)
    input_stereo_width = WwiseProperty[float]("InputStereoWidth", float)
    input_threshold = WwiseProperty[float]("InputThreshold", float)
    lfe_level = WwiseProperty[float]("LFELevel", float)
    pre_delay = WwiseProperty[float]("PreDelay", float)
    rear_level = WwiseProperty[float]("RearLevel", float)
    sound_engine_block_size = WwiseProperty[EConvolutionReverbBlockSize](
        "SoundEngineBlockSize", EConvolutionReverbBlockSize)
    sound_engine_sample_rate_default = WwiseProperty[int]("SoundEngineSampleRateDefault", int)
    sound_engine_sample_rate_mac = WwiseProperty[int]("SoundEngineSampleRateMac", int)
    sound_engine_sample_rate_ios = WwiseProperty[int]("SoundEngineSampleRateiOS", int)
    stereo_width = WwiseProperty[float]("StereoWidth", float)
    wet_level = WwiseProperty[float]("WetLevel", float)



# Copyright 2025 Matheus Vilano
# SPDX-License-Identifier: Apache-2.0

from pywwise.descriptors import WwiseProperty
from pywwise.enums import EGuitarDistortionType, EGuitarDistortionFilterType
from pywwise.objects.abc import WwiseObject


class WwiseGuitarDistortion(WwiseObject):
    """
    https://www.audiokinetic.com/en/library/edge/?source=SDK&id=wwiseobject_effect_wwise_guitar_distortion.html \n
    A class serving as an interface for getting/setting properties on Wwise objects. This type specifically targets
    the class represented by `EObjectType.WWISE_GUITAR_DISTORTION`.
    """
    distortion_drive = WwiseProperty[float]("DistortionDrive", float)
    distortion_tone = WwiseProperty[float]("DistortionTone", float)
    distortion_type = WwiseProperty[EGuitarDistortionType]("DistortionType", EGuitarDistortionType)
    output_level = WwiseProperty[float]("OutputLevel", float)
    post_eq_band1_enable = WwiseProperty[bool]("PostEQBand1Enable", bool)
    post_eq_band1_type = WwiseProperty[EGuitarDistortionFilterType]("PostEQBand1FilterType", EGuitarDistortionFilterType)
    post_eq_band1_frequency = WwiseProperty[float]("PostEQBand1Frequency", float)
    post_eq_band1_gain = WwiseProperty[float]("PostEQBand1Gain", float)
    post_eq_band1_q = WwiseProperty[float]("PostEQBand1QFactor", float)
    post_eq_band2_enable = WwiseProperty[bool]("PostEQBand2Enable", bool)
    post_eq_band2_type = WwiseProperty[EGuitarDistortionFilterType]("PostEQBand2FilterType", EGuitarDistortionFilterType)
    post_eq_band2_frequency = WwiseProperty[float]("PostEQBand2Frequency", float)
    post_eq_band2_gain = WwiseProperty[float]("PostEQBand2Gain", float)
    post_eq_band2_q = WwiseProperty[float]("PostEQBand2QFactor", float)
    post_eq_band3_enable = WwiseProperty[bool]("PostEQBand3Enable", bool)
    post_eq_band3_type = WwiseProperty[EGuitarDistortionFilterType]("PostEQBand3FilterType", EGuitarDistortionFilterType)
    post_eq_band3_frequency = WwiseProperty[float]("PostEQBand3Frequency", float)
    post_eq_band3_gain = WwiseProperty[float]("PostEQBand3Gain", float)
    post_eq_band3_q = WwiseProperty[float]("PostEQBand3QFactor", float)
    pre_eq_band1_enable = WwiseProperty[bool]("PreEQBand1Enable", bool)
    pre_eq_band1_type = WwiseProperty[EGuitarDistortionFilterType]("PreEQBand1FilterType", EGuitarDistortionFilterType)
    pre_eq_band1_frequency = WwiseProperty[float]("PreEQBand1Frequency", float)
    pre_eq_band1_gain = WwiseProperty[float]("PreEQBand1Gain", float)
    pre_eq_band1_q = WwiseProperty[float]("PreEQBand1QFactor", float)
    pre_eq_band2_enable = WwiseProperty[bool]("PreEQBand2Enable", bool)
    pre_eq_band2_type = WwiseProperty[EGuitarDistortionFilterType]("PreEQBand2FilterType", EGuitarDistortionFilterType)
    pre_eq_band2_frequency = WwiseProperty[float]("PreEQBand2Frequency", float)
    pre_eq_band2_gain = WwiseProperty[float]("PreEQBand2Gain", float)
    pre_eq_band2_q = WwiseProperty[float]("PreEQBand2QFactor", float)
    pre_eq_band3_enable = WwiseProperty[bool]("PreEQBand3Enable", bool)
    pre_eq_band3_type = WwiseProperty[EGuitarDistortionFilterType]("PreEQBand3FilterType", EGuitarDistortionFilterType)
    pre_eq_band3_frequency = WwiseProperty[float]("PreEQBand3Frequency", float)
    pre_eq_band3_gain = WwiseProperty[float]("PreEQBand3Gain", float)
    pre_eq_band3_q = WwiseProperty[float]("PreEQBand3QFactor", float)
    rectification = WwiseProperty[float]("Rectification", float)
    wet_dry_mix = WwiseProperty[float]("WetDryMix", float)
    


# Copyright 2025 Matheus Vilano
# SPDX-License-Identifier: Apache-2.0

from pywwise.descriptors import WwiseProperty
from pywwise.enums import (EActionNamePosition, ECaseStyleSimple, EColour, EFftWindowSize, EFilterBehaviour,
                           ELineEnding, ESoundBankDefinitionFormat)
from pywwise.objects.abc import WwiseObject
from pywwise.primitives import ProjectPath


class Project(WwiseObject):
    """
    https://www.audiokinetic.com/en/library/edge/?source=SDK&id=wwiseobject_project.html \n
    A class serving as an interface for getting/setting properties on Wwise objects. This type specifically targets
    the class represented by `EObjectType.PROJECT`.
    """
    always_save_media_ids_file = WwiseProperty[bool]("AlwaysSaveMediaIdsFile", bool)
    auto_detect_fft_window_size = WwiseProperty[EFftWindowSize]("AutoDetectFFTWindowSize", EFftWindowSize)
    auto_detect_threshold_high = WwiseProperty[float]("AutoDetectThresholdHigh", float)
    auto_detect_threshold_low = WwiseProperty[float]("AutoDetectThresholdLow", float)
    auto_detect_threshold_medium = WwiseProperty[float]("AutoDetectThresholdMedium", float)
    auto_sound_bank_all_events = WwiseProperty[bool]("AutoSoundBankAllEvents", bool)
    auto_sound_bank_enabled = WwiseProperty[bool]("AutoSoundBankEnabled", bool)
    colour = WwiseProperty[EColour]("Color", EColour)
    comm_port_discovery_broadcast = WwiseProperty[int]("CommPortDiscoveryBroadcast", int)
    comm_port_discovery_response = WwiseProperty[int]("CommPortDiscoveryResponse", int)
    comm_serial_post_base = WwiseProperty[int]("CommSerialPostBase", int)
    copy_loose_streamed_media = WwiseProperty[bool]("CopyLooseStreamedMedia", bool)
    default_language = WwiseProperty[str]("DefaultLanguage", str)
    event_action_name_position = WwiseProperty[EActionNamePosition]("EventActionNamePosition", EActionNamePosition)
    event_name_case_type = WwiseProperty[ECaseStyleSimple]("EventNameCaseType", ECaseStyleSimple)
    event_name_modify_case = WwiseProperty[int]("EventNameModifyCase", int)
    external_sources_input_path = WwiseProperty[ProjectPath]("ExternalSourcesInputPath", ProjectPath)
    external_sources_output_path = WwiseProperty[ProjectPath]("ExternalSourcesOutputPath", ProjectPath)
    filter_behaviour = WwiseProperty[EFilterBehaviour]("FilterBehavior", EFilterBehaviour)
    garbage_collect_wave_analysis = WwiseProperty[bool]("GarbageCollectWaveAnalysis", bool)
    generate_main_sound_bank = WwiseProperty[bool]("GenerateMainSoundBank", bool)
    generate_multiple_banks = WwiseProperty[bool]("GenerateMultipleBanks", bool)
    generate_sound_bank_json = WwiseProperty[bool]("GenerateSoundBankJSON", bool)
    generate_sound_bank_xml = WwiseProperty[bool]("GenerateSoundBankXML", bool)
    global_voices_limit = WwiseProperty[int]("GlobalVoicesLimit", int)
    license_key = WwiseProperty[str]("LicenseKey", str)
    line_ending = WwiseProperty[ELineEnding]("LineEnding", ELineEnding)
    max_dangerous_virtual_voices = WwiseProperty[int]("MaxDangerousVirtualVoices", int)
    max_messages_per_message_id = WwiseProperty[int]("MaxMessagesPerMessageId", int)
    media_auto_bank_sub_folders = WwiseProperty[bool]("MediaAutoBankSubFolders", bool)
    media_ids_in_single_file = WwiseProperty[bool]("MediaIDsInSingleFile", bool)
    override_colour = WwiseProperty[bool]("OverrideColor", bool)
    remove_unused_generated_files = WwiseProperty[bool]("RemoveUnusedGeneratedFiles", bool)
    sound_bank_allow_exceeding_sb = WwiseProperty[bool]("SoundBankAllowExceedingSB", bool)
    sound_bank_definition_file_format = WwiseProperty[ESoundBankDefinitionFormat](
        "SoundBankDefinitionFileFormat", ESoundBankDefinitionFormat)
    sound_bank_generate_definition_file = WwiseProperty[bool]("SoundBankGenerateDefinitionFile", bool)
    sound_bank_generate_estimated_duration = WwiseProperty[bool]("SoundBankGenerateEstimatedDuration", bool)
    sound_bank_generate_header_file = WwiseProperty[bool]("SoundBankGenerateHeaderFile", bool)
    sound_bank_generate_max_attenuation_info = WwiseProperty[bool]("SoundBankGenerateMaxAttenuationInfo", bool)
    sound_bank_generate_print_colour = WwiseProperty[bool]("SoundBankGeneratePrintColor", bool)
    sound_bank_generate_print_guid = WwiseProperty[bool]("SoundBankGeneratePrintGUID", bool)
    sound_bank_generate_print_path = WwiseProperty[bool]("SoundBankGeneratePrintPath", bool)
    sound_bank_generate_readable_file = WwiseProperty[bool]("SoundBankGenerateReadableFile", bool)
    sound_bank_header_file_path = WwiseProperty[ProjectPath]("SoundBankHeaderFilePath", ProjectPath)
    sound_bank_include_soundbank_names_strings = WwiseProperty[bool]("SoundBankIncludeSoundbankNamesStrings", bool)
    sound_bank_paths = WwiseProperty[str]("SoundBankPaths", str)
    sound_bank_post_generate_custom_cmd_description = WwiseProperty[str](
        "SoundBankPostGenerateCustomCmdDescription", str)
    sound_bank_post_generate_custom_cmd_lines = WwiseProperty[str]("SoundBankPostGenerateCustomCmdLines", str)
    sound_bank_pre_generate_custom_cmd_description = WwiseProperty[str]("SoundBankPreGenerateCustomCmdDescription", str)
    sound_bank_pre_generate_custom_cmd_lines = WwiseProperty[str]("SoundBankPreGenerateCustomCmdLines", str)
    sound_bank_update_audio_files = WwiseProperty[bool]("SoundBankUpdateAudioFiles", bool)
    source_control_generated_files = WwiseProperty[bool]("SourceControlGeneratedFiles", bool)
    use_action_name_for_event = WwiseProperty[bool]("UseActionNameForEvent", bool)
    use_default_language = WwiseProperty[bool]("UseDefaultLanguage", bool)
    use_max_dangerous_virtual_voices = WwiseProperty[bool]("UseMaxDangerousVirtualVoices", bool)
    use_max_messages_per_message_id = WwiseProperty[bool]("UseMaxMessagesPerMessageId", bool)
    volume_threshold = WwiseProperty[float]("VolumeThreshold", float)
    wwise_console_load_user_settings = WwiseProperty[bool]("WwiseConsoleLoadUserSettings", bool)
    wwise_version_when_created = WwiseProperty[str]("WwiseVersionWhenCreated", str)



# Copyright 2024 Matheus Vilano
# SPDX-License-Identifier: Apache-2.0

from waapi import WaapiClient as _WaapiClient


class Undo:
    """ak.wwise.core.undo"""
    
    def __init__(self, client: _WaapiClient):
        """
        Constructor.
        :param client: The WAAPI client to use.
        """
        self._client = client
    
    def begin_group(self) -> bool:
        """
        https://www.audiokinetic.com/library/edge/?source=SDK&id=ak_wwise_core_undo_begingroup.html \n
        Begins an undo group. Make sure to call ak.wwise.core.undo.endGroup exactly once for every
        ak.wwise.core.beginUndoGroup call you make. Calls to `ak.wwise.core.undo.begin_group` can be nested.
        When closing a WAMP session, a check is made to ensure that all undo groups are closed. If not,
        a cancelGroup is called for each of the groups still open.
        :return: Whether the call was successful.
        """
        return self._client.call("ak.wwise.core.undo.beginGroup") is not None
    
    def cancel_group(self, undo: bool = False) -> bool:
        """
        https://www.audiokinetic.com/library/edge/?source=SDK&id=ak_wwise_core_undo_cancelgroup.html \n
        Cancels the last undo group.
        :param undo: Specify if the operations are undone.
        :return: Whether the call was successful.
        """
        return self._client.call("ak.wwise.core.undo.cancelGroup", {"undo": undo}) is not None
    
    def end_group(self, display_name: str) -> bool:
        """
        https://www.audiokinetic.com/library/edge/?source=SDK&id=ak_wwise_core_undo_endgroup.html \n
        Ends the last undo group.
        :param display_name: The name that is displayed in the history for this undo group.
        :return: Whether the call was successful.
        """
        return self._client.call("ak.wwise.core.undo.endGroup", {"displayName": display_name}) is not None
    
    def redo(self) -> bool:
        """
        https://www.audiokinetic.com/library/edge/?source=SDK&id=ak_wwise_core_undo_redo.html \n
        Redoes the last operation in the Undo stack.
        :return: Whether the call was successful.
        """
        return self._client.call("ak.wwise.core.undo.redo") is not None
    
    def undo(self) -> bool:
        """
        https://www.audiokinetic.com/library/edge/?source=SDK&id=ak_wwise_core_undo_undo.html \n
        Undoes the last operation in the Undo stack.
        :return: Whether the call was successful.
        """
        return self._client.call("ak.wwise.core.undo.undo") is not None



# Copyright 2025 Matheus Vilano
# SPDX-License-Identifier: Apache-2.0

from pywwise.descriptors import WwiseProperty
from pywwise.enums import EColour, EMusicCueType
from pywwise.objects.abc import WwiseObject


class MusicCue(WwiseObject):
    """
    https://www.audiokinetic.com/en/library/edge/?source=SDK&id=wwiseobject_musiccue.html \n
    A class serving as an interface for getting/setting properties on Wwise objects. This type specifically targets
    the class represented by `EObjectType.MUSIC_CUE`.
    """
    colour = WwiseProperty[EColour]("Color", EColour)
    cue_type = WwiseProperty[EMusicCueType]("CueType", EMusicCueType)
    override_colour = WwiseProperty[bool]("OverrideColor", bool)
    time_ms = WwiseProperty[float]("TimeMs", float)



# Copyright 2025 Matheus Vilano
# SPDX-License-Identifier: Apache-2.0

from __future__ import annotations

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from pywwise.objects.types.rtpc import Rtpc

from pywwise.descriptors import WwiseProperty
from pywwise.enums import EColour, EModulatorScope, EWaveformInt
from pywwise.objects.abc import WwiseObject


class ModulatorLfo(WwiseObject):
    """
    https://www.audiokinetic.com/en/library/edge/?source=SDK&id=wwiseobject_modulatorlfo.html \n
    A class serving as an interface for getting/setting properties on Wwise objects. This type specifically targets
    the class represented by `EObjectType.MODULATOR_LFO`.
    """
    colour = WwiseProperty[EColour]("Color", EColour)
    lfo_attack = WwiseProperty[float]("LfoAttack", float)
    lfo_depth = WwiseProperty[float]("LfoDepth", float)
    lfo_frequency = WwiseProperty[float]("LfoFrequency", float)
    lfo_initial_phase = WwiseProperty[float]("LfoInitialPhase", float)
    lfo_pwm = WwiseProperty[float]("LfoPWM", float)
    lfo_smoothing = WwiseProperty[float]("LfoSmoothing", float)
    lfo_waveform = WwiseProperty[EWaveformInt]("LfoWaveform", EWaveformInt)
    modulator_scope = WwiseProperty[EModulatorScope]("ModulatorScope", EModulatorScope)
    override_colour = WwiseProperty[bool]("OverrideColor", bool)
    rtpc: WwiseProperty[tuple[Rtpc, ...]] = WwiseProperty("RTPC", tuple)  # Using `:` to avoid circular imports.
